@@ -24,7 +24,7 @@
 #include <config.h>
 extern "C" {
   #include <glib/gi18n.h>
-  #include <glib/gfileutils.h> // for g_build_filename
+  #include <glib.h> // for g_build_filename
 }
 #include <pan/general/debug.h>
 #include <pan/general/file-util.h>
@@ -33,6 +33,11 @@ extern "C" {
 #include <pan/general/time-elapsed.h>
 #include "data-impl.h"
 
+#ifdef HAVE_GKR
+  #include <gnome-keyring-1/gnome-keyring.h>
+  #include <gnome-keyring-1/gnome-keyring-memory.h>
+#endif
+
 using namespace pan;
 
 /**
@@ -41,6 +46,7 @@ using namespace pan;
 
 namespace
 {
+
   std::string get_cache_path ()
   {
     char * pch (g_build_filename (file::get_pan_home().c_str(), "article-cache", NULL));
@@ -49,16 +55,29 @@ namespace
     g_free (pch);
     return path;
   }
+
+  std::string get_encode_cache_path ()
+  {
+    char * pch (g_build_filename (file::get_pan_home().c_str(), "encode-cache", NULL));
+    file :: ensure_dir_exists (pch);
+    std::string path (pch);
+    g_free (pch);
+    return path;
+  }
+
 }
 
-DataImpl :: DataImpl (bool unit_test, int cache_megs, DataIO * io):
+DataImpl :: DataImpl (const StringView& cache_ext, bool unit_test, int cache_megs, DataIO * io):
   ProfilesImpl (*io),
-  _cache (get_cache_path(), cache_megs),
+  _cache (get_cache_path(), cache_ext, cache_megs),
+  _encode_cache (get_encode_cache_path(), cache_megs),
+  _certstore(*this),
   _unit_test (unit_test),
   _data_io (io),
   _descriptions_loaded (false),
   newsrc_autosave_id (0),
   newsrc_autosave_timeout (0)
+
 {
   rebuild_backend ();
 }
@@ -94,6 +113,7 @@ DataImpl :: rebuild_backend ()
 DataImpl :: ~DataImpl ()
 {
   save_state ();
+
 }
 
 void
@@ -106,3 +126,51 @@ DataImpl :: save_state ()
     save_newsrc_files (*_data_io);
   }
 }
+
+#ifdef HAVE_GKR
+GnomeKeyringResult
+DataImpl :: password_encrypt (const PasswordData& pw)
+{
+//  g_return_val_if_fail (pw, GNOME_KEYRING_RESULT_NO_KEYRING_DAEMON);
+
+  return (
+    gnome_keyring_store_password_sync (
+      GNOME_KEYRING_NETWORK_PASSWORD,
+      GNOME_KEYRING_DEFAULT,
+      _("Pan Newsreader's server passwords"),
+      pw.pw.str,
+      "user", pw.user.str,
+      "server", pw.server.c_str(),
+      NULL)
+    );
+
+}
+
+// TODO use gnome_keyring_memory_new etc
+GnomeKeyringResult
+DataImpl :: password_decrypt (PasswordData& pw) const
+{
+
+  gchar* pwd = NULL;
+//  g_return_val_if_fail (pw, GNOME_KEYRING_RESULT_NO_KEYRING_DAEMON);
+
+  GnomeKeyringResult ret =
+    gnome_keyring_find_password_sync (
+    GNOME_KEYRING_NETWORK_PASSWORD,
+    &pwd,
+    "user", pw.user.str,
+    "server", pw.server.c_str(),
+    NULL);
+
+  std::string tmp;
+  if (pwd)
+  { tmp = pwd;
+    gnome_keyring_free_password(pwd);
+  }
+  pw.pw = tmp;
+
+  return (pwd ? GNOME_KEYRING_RESULT_OK : GNOME_KEYRING_RESULT_DENIED) ;
+}
+#endif
+
+
