@@ -28,6 +28,7 @@
 #include <set>
 #include <vector>
 
+#include <glib.h>
 extern "C" {
   #include <unistd.h>
   #include <glib/gi18n.h>
@@ -154,6 +155,20 @@ DataImpl :: load_newsrc (const Quark       & server,
   }
 }
 
+namespace {
+  inline std::string newsrc_fn(const std::string &base)
+  {
+    const char *fn = base.c_str();
+    if (g_path_is_absolute(fn))
+      return base;
+    const char *ph = file::get_pan_home().c_str();
+    char *temp = g_build_filename(ph, fn, NULL);
+    std::string out(temp);
+    g_free(temp);
+    return out;
+  }
+}
+
 void
 DataImpl :: load_newsrc_files (const DataIO& data_io)
 {
@@ -165,7 +180,7 @@ DataImpl :: load_newsrc_files (const DataIO& data_io)
 
   foreach_const (servers_t, _servers, sit) {
     const Quark key (sit->first);
-    const std::string filename (sit->second.newsrc_filename);
+    const std::string filename = newsrc_fn (sit->second.newsrc_filename);
     if (file::file_exists (filename.c_str())) {
       LineReader * in (data_io.read_file (filename));
       load_newsrc (key, in, s, u);
@@ -193,6 +208,9 @@ DataImpl :: load_newsrc_files (const DataIO& data_io)
 void
 DataImpl :: save_newsrc_files (DataIO& data_io) const
 {
+  if (!in_newsrc_cb && newsrc_autosave_id)
+    g_source_remove( newsrc_autosave_id );
+
   if (_unit_test)
     return;
 
@@ -209,7 +227,7 @@ DataImpl :: save_newsrc_files (DataIO& data_io) const
     const Quark& server (sit->first);
 
     // write this server's newsrc
-    const std::string& filename (sit->second.newsrc_filename);
+    const std::string filename = newsrc_fn (sit->second.newsrc_filename);
     std::ostream& out (*data_io.write_file (filename));
     std::string newsrc_string;
     alpha_groups_t::const_iterator sub_it (_subscribed.begin());
@@ -360,7 +378,7 @@ DataImpl :: load_group_xovers (const DataIO& data_io)
 
         while (line.pop_token (xover))
           if (xover.pop_token(servername,':'))
-            g[servername]._xover_high = strtoul (xover.str, NULL, 10);
+            g[servername]._xover_high = g_ascii_strtoull (xover.str, NULL, 10);
       }
     }
   }
@@ -457,11 +475,11 @@ DataImpl :: save_group_xovers (DataIO& data_io) const
 *****
 ****/
 
-unsigned long
+uint64_t
 DataImpl :: get_xover_high (const Quark  & groupname,
                             const Quark  & servername) const
 {
-  unsigned long high (0ul);
+  uint64_t high (0ul);
   const ReadGroup::Server * rgs (find_read_group_server (groupname, servername));
   if (rgs)
     high = rgs->_xover_high;
@@ -471,7 +489,7 @@ DataImpl :: get_xover_high (const Quark  & groupname,
 void
 DataImpl :: set_xover_high (const Quark & group,
                             const Quark & server,
-                            const unsigned long high)
+                            const uint64_t high)
 {
   //std::cerr << LINE_ID << "setting " << get_server_address(server) << ':' << group << " xover high to " << high << std::endl;
   ReadGroup::Server& rgs (_read_groups[group][server]);
