@@ -72,56 +72,80 @@ DataImpl :: add_new_server ()
 
   // add it to the _servers map and give it a default filename
   std::ostringstream o;
-  o << file::get_pan_home() << G_DIR_SEPARATOR << "newsrc-" << new_server;
+  o << "newsrc-" << new_server;
   _servers[new_server].newsrc_filename = o.str ();
   return new_server;
 }
 
-DataImpl :: Server*
+Data :: Server*
 DataImpl :: find_server (const Quark& server)
 {
   Server * retval (0);
+
   servers_t::iterator it (_servers.find (server));
   if (it != _servers.end())
     retval = &it->second;
   return retval;
 }
 
-const DataImpl :: Server*
+const Data :: Server*
 DataImpl :: find_server (const Quark& server) const
 {
   const Server * retval (0);
+
   servers_t::const_iterator it (_servers.find (server));
   if (it != _servers.end())
     retval = &it->second;
   return retval;
 }
 
+bool
+DataImpl :: find_server_by_hn (const Quark& server, Quark& setme) const
+{
+  foreach_const(servers_t, _servers, it)
+    if (it->second.host == server.c_str()) { setme = it->first; return true; }
+  return false;
+}
+
 void
 DataImpl :: set_server_article_expiration_age  (const Quark  & server,
                                                 int            days)
 {
   Server * s (find_server (server));
-  assert (s != 0);
+  assert (s);
 
   s->article_expiration_age = std::max (0, days);
 
-  save_server_properties (*_data_io);
 }
 
-  
 void
 DataImpl :: set_server_auth (const Quark       & server,
                              const StringView  & username,
                              const StringView  & password)
 {
   Server * s (find_server (server));
-  assert (s != 0);
+  assert (s);
 
   s->username = username;
+#ifndef HAVE_GKR
   s->password = password;
+#else
+  PasswordData pw;
+  pw.server = s->host;
+  pw.user = username;
+  pw.pw = password;
+  password_encrypt(pw);
+#endif
 
-  save_server_properties (*_data_io);
+}
+
+void
+DataImpl :: set_server_trust  (const Quark   & server,
+                               const int       setme)
+{
+  Server * s (find_server (server));
+  assert (s);
+  s->trust = setme;
 }
 
 void
@@ -130,10 +154,10 @@ DataImpl :: set_server_addr (const Quark       & server,
                              int                 port)
 {
   Server * s (find_server (server));
-  assert (s != 0);
+  assert (s);
   s->host = host;
   s->port = port;
-  save_server_properties (*_data_io);
+
 }
 
 
@@ -142,9 +166,9 @@ DataImpl :: set_server_limits (const Quark   & server,
                                int             max_connections)
 {
   Server * s (find_server (server));
-  assert (s != 0);
+  assert (s);
   s->max_connections = max_connections;
-  save_server_properties (*_data_io);
+
 }
 
 void
@@ -152,44 +176,147 @@ DataImpl :: set_server_rank (const Quark   & server,
                              int             rank)
 {
   Server * s (find_server (server));
-  assert (s != 0);
+  assert (s);
   s->rank = rank;
-  save_server_properties (*_data_io);
+
 }
 
+void
+DataImpl :: set_server_ssl_support (const Quark   & server,
+                                    int             ssl)
+{
+  Server * s (find_server (server));
+  assert (s);
+  s->ssl_support = ssl;
+
+}
+
+void
+DataImpl :: set_server_cert  (const Quark   & server,
+                              const StringView & cert)
+{
+
+  Server * s (find_server (server));
+  assert (s);
+  s->cert = cert;
+
+}
+
+void
+DataImpl :: save_server_info (const Quark& server)
+{
+  Server * s (find_server (server));
+  assert (s);
+  save_server_properties (*_data_io);
+
+}
+
+
 bool
 DataImpl :: get_server_auth (const Quark   & server,
                              std::string   & setme_username,
                              std::string   & setme_password) const
 {
   const Server * s (find_server (server));
-  const bool found (s != 0);
+  bool found (s);
   if (found) {
     setme_username = s->username;
+#ifndef HAVE_GKR
     setme_password = s->password;
+#else
+    PasswordData pw;
+    pw.server = s->host;
+    pw.user = s->username;
+    GnomeKeyringResult res = password_decrypt(pw);
+    switch (res)
+    {
+      case GNOME_KEYRING_RESULT_NO_MATCH:
+        Log::add_info_va(_("There seems to be no password set for server %s."), s->host.c_str());
+        break;
+
+      case GNOME_KEYRING_RESULT_NO_KEYRING_DAEMON:
+        Log::add_urgent_va (_("GNOME Keyring denied access to the passwords."), s->host.c_str());
+        break;
+
+      case GNOME_KEYRING_RESULT_OK:
+        setme_password.assign(pw.pw.str, pw.pw.len);
+        break;
+
+      default:
+        break;
+    }
+#endif
   }
+
+  return found;
+
+}
+
+bool
+DataImpl :: get_server_trust (const Quark   & server, int& setme) const
+{
+  const Server * s (find_server (server));
+  const bool found (s);
+  if (found) {
+    setme = s->trust;
+  }
+
   return found;
 }
-                                                                                             
+
 bool
 DataImpl :: get_server_addr (const Quark   & server,
                              std::string   & setme_host,
                              int           & setme_port) const
 {
   const Server * s (find_server (server));
-  const bool found (s != 0);
+  const bool found (s);
   if (found) {
     setme_host = s->host;
     setme_port = s->port;
   }
+
   return found;
+
 }
 
 std::string
 DataImpl :: get_server_address (const Quark& server) const
 {
+  std::string str;
   const Server * s (find_server (server));
-  return std::string (s ? s->host : "");
+  if (s) {
+    std::ostringstream x(s->host,std::ios_base::ate);
+    x << ":" << s->port;
+    str = x.str();
+  }
+
+  return str;
+
+}
+
+bool
+DataImpl :: get_server_ssl_support (const Quark & server) const
+{
+  bool retval (false);
+  const Server * s (find_server (server));
+  if (s)
+    retval = (s->ssl_support != 0);
+
+  return retval;
+
+}
+
+std::string
+DataImpl :: get_server_cert (const Quark & server) const
+{
+  std::string str;
+  const Server * s (find_server (server));
+  if (s)
+    str = s->cert;
+
+  return str;
+
 }
 
 int
@@ -197,9 +324,11 @@ DataImpl :: get_server_limits (const Quark & server) const
 {
   int retval (2);
   const Server * s (find_server (server));
-  if (s != 0)
+  if (s)
     retval = s->max_connections;
+
   return retval;
+
 }
 
 int
@@ -207,9 +336,11 @@ DataImpl :: get_server_rank (const Quark & server) const
 {
   int retval (1);
   const Server * s (find_server (server));
-  if (s != 0)
+  if (s)
     retval = s->rank;
+
   return retval;
+
 }
 
 int
@@ -217,11 +348,14 @@ DataImpl :: get_server_article_expiration_age  (const Quark  & server) const
 {
   int retval (31);
   const Server * s (find_server (server));
-  if (s != 0)
+  if (s)
     retval = s->article_expiration_age;
+
   return retval;
+
 }
 
+
 /***
 ****
 ***/
@@ -265,7 +399,7 @@ namespace
 
   void text (GMarkupParseContext *context    UNUSED,
              const gchar         *text,
-             gsize                text_len,  
+             gsize                text_len,
              gpointer             user_data,
              GError             **error      UNUSED)
   {
@@ -311,11 +445,18 @@ DataImpl :: load_server_properties (const DataIO& source)
     keyvals_t kv (it->second);
     s.host = kv["host"];
     s.username = kv["username"];
+#ifndef HAVE_GKR
     s.password = kv["password"];
-    s.port = to_int (kv["port"], 119);
+#endif
+    s.port = to_int (kv["port"], STD_NNTP_PORT);
     s.max_connections = to_int (kv["connection-limit"], 2);
     s.article_expiration_age = to_int(kv["expire-articles-n-days-old"], 31);
     s.rank = to_int(kv["rank"], 1);
+    int ssl(to_int(kv["use-ssl"], 0));
+    s.ssl_support = ssl;
+    s.cert = kv["cert"];
+    int trust(to_int(kv["trust"], 0));
+    s.trust = trust;
     s.newsrc_filename = kv["newsrc"];
     if (s.newsrc_filename.empty()) { // set a default filename
       std::ostringstream o;
@@ -324,7 +465,6 @@ DataImpl :: load_server_properties (const DataIO& source)
     }
   }
 
-  save_server_properties (*const_cast<DataIO*>(&source));
 }
 
 namespace
@@ -360,15 +500,21 @@ DataImpl :: save_server_properties (DataIO& data_io) const
   *out << indent(depth++) << "<server-properties>\n";
   foreach_const (alpha_quarks_t, servers, it) {
     const Server* s (find_server (*it));
+    std::string user, pass;
+    get_server_auth(*it, user, pass);
     *out << indent(depth++) << "<server id=\"" << escaped(it->to_string()) << "\">\n";
     *out << indent(depth) << "<host>" << escaped(s->host) << "</host>\n"
          << indent(depth) << "<port>" << s->port << "</port>\n"
-         << indent(depth) << "<username>" << escaped(s->username) << "</username>\n"
-         << indent(depth) << "<password>" << escaped(s->password) << "</password>\n"
+         << indent(depth) << "<username>" << escaped(user) << "</username>\n"
+         << indent(depth) << "<password>" << escaped(pass) << "</password>\n"
          << indent(depth) << "<expire-articles-n-days-old>" << s->article_expiration_age << "</expire-articles-n-days-old>\n"
          << indent(depth) << "<connection-limit>" << s->max_connections << "</connection-limit>\n"
          << indent(depth) << "<newsrc>" << s->newsrc_filename << "</newsrc>\n"
-         << indent(depth) << "<rank>" << s->rank << "</rank>\n";
+         << indent(depth) << "<rank>" << s->rank << "</rank>\n"
+         << indent(depth) << "<use-ssl>" << s->ssl_support << "</use-ssl>\n"
+         << indent(depth) << "<trust>" << s->trust << "</trust>\n"
+         << indent(depth) << "<cert>"    << s->cert << "</cert>\n";
+
     *out << indent(--depth) << "</server>\n";
   }
   *out << indent(--depth) << "</server-properties>\n";
