@@ -23,11 +23,13 @@
 #include <map>
 #include <vector>
 extern "C" {
-  #include <glib/gtypes.h> // for guint64
+  #include <glib.h> // for guint64
 }
 #include <pan/general/string-view.h>
 #include <pan/general/quark.h>
 
+#include <pan/usenet-utils/gpg.h>
+
 extern "C"
 {
   typedef struct _GMimeMessage GMimeMessage;
@@ -36,6 +38,7 @@ extern "C"
 
 namespace pan
 {
+
   class Article;
   class StringView;
 
@@ -58,7 +61,7 @@ namespace pan
   {
     public:
 
-      ArticleCache (const StringView& path, size_t max_megs=10);
+      ArticleCache (const StringView& path, const StringView& extension, size_t max_megs=10);
       ~ArticleCache ();
 
       typedef std::vector<Quark> mid_sequence_t;
@@ -69,9 +72,11 @@ namespace pan
       void release (const mid_sequence_t& mids);
       void resize ();
       void clear ();
-
-      GMimeMessage* get_message (const mid_sequence_t&);
-
+#ifdef HAVE_GMIME_CRYPTO
+      GMimeMessage* get_message (const mid_sequence_t&, GPGDecErr&) const;
+#else
+      GMimeMessage* get_message (const mid_sequence_t&) const;
+#endif
       typedef std::vector<std::string> strings_t;
       strings_t get_filenames (const mid_sequence_t&);
 
@@ -86,10 +91,17 @@ namespace pan
       void add_listener (Listener * l) { _listeners.insert(l); }
       void remove_listener (Listener * l) { _listeners.erase(l); }
 
+    public:
+      void set_max_megs (size_t value) { _max_megs = value; }
+      void set_msg_extension (const std::string& s) { msg_extension = s; }
+      const std::string& get_msg_extension () const { return msg_extension; }
+
     private:
 
       std::map<Quark,int> _locks;
 
+      std::string msg_extension;
+
       struct MsgInfo {
         Quark _message_id;
         size_t _size;
@@ -109,7 +121,7 @@ namespace pan
       };
 
       std::string _path;
-      size_t _max_megs;
+      size_t _max_megs; // changeable via prefs
       guint64 _current_bytes;
 
       typedef std::set<Listener*> listeners_t;
@@ -123,6 +135,10 @@ namespace pan
       char* get_filename (char* buf, int buflen, const Quark& mid) const;
       GMimeStream* get_message_file_stream (const Quark& mid) const;
       GMimeStream* get_message_mem_stream (const Quark& mid) const;
+
+      int filename_to_message_id (char * buf, int len, const char * basename);
+      char* message_id_to_filename (char * buf, int len, const StringView& mid) const;
+
   };
 }
 
