@@ -20,19 +20,28 @@
 #include <config.h>
 extern "C" {
   #include <glib/gi18n.h>
-  #include <gtk/gtk.h>
+  #include "gtk-compat.h"
 }
+
+#include <pan/general/macros.h>
 #include <pan/general/debug.h>
+#include <pan/data-impl/data-impl.h>
 #include <pan/tasks/task-xover.h>
 #include <pan/icons/pan-pixbufs.h>
 #include "actions.h"
 #include "pad.h"
 #include "gui.h"
 
+namespace pan
+{
+  std::map<std::string,std::string> action_trans;
+}
+
 using pan::PanUI;
 
-namespace
+namespace pan
 {
+
   PanUI * pan_ui (0);
   Prefs * prefs (0);
 
@@ -46,24 +55,26 @@ namespace
   {
     { icon_article_read, "ICON_ARTICLE_READ" },
     { icon_article_unread, "ICON_ARTICLE_UNREAD" },
-    { icon_compose_followup, "ICON_COMPOSE_FOLLOWUP" }, 
-    { icon_compose_post, "ICON_COMPOSE_POST" }, 
-    { icon_disk, "ICON_DISK" }, 
-    { icon_filter_only_attachments, "ICON_ONLY_ATTACHMENTS" }, 
-    { icon_filter_only_cached, "ICON_ONLY_CACHED" }, 
-    { icon_filter_only_me, "ICON_ONLY_ME" }, 
-    { icon_filter_only_unread, "ICON_ONLY_UNREAD" }, 
-    { icon_filter_only_watched, "ICON_ONLY_WATCHED" }, 
-    { icon_get_dialog, "ICON_GET_DIALOG" }, 
-    { icon_get_selected, "ICON_GET_SELECTED" }, 
-    { icon_get_subscribed, "ICON_GET_SUBSCRIBED" }, 
-    { icon_read_group, "ICON_READ_GROUP" }, 
-    { icon_read_more, "ICON_READ_MORE" }, 
-    { icon_read_less, "ICON_READ_LESS" }, 
-    { icon_read_unread_article, "ICON_READ_UNREAD_ARTICLE" }, 
-    { icon_read_unread_thread, "ICON_READ_UNREAD_THREAD" }, 
-    { icon_score, "ICON_SCORE" }, 
-    { icon_search_pulldown, "ICON_SEARCH_PULLDOWN" }
+    { icon_compose_followup, "ICON_COMPOSE_FOLLOWUP" },
+    { icon_compose_post, "ICON_COMPOSE_POST" },
+    { icon_disk, "ICON_DISK" },
+    { icon_filter_only_attachments, "ICON_ONLY_ATTACHMENTS" },
+    { icon_filter_only_cached, "ICON_ONLY_CACHED" },
+    { icon_filter_only_me, "ICON_ONLY_ME" },
+    { icon_filter_only_unread, "ICON_ONLY_UNREAD" },
+    { icon_filter_only_watched, "ICON_ONLY_WATCHED" },
+    { icon_get_dialog, "ICON_GET_DIALOG" },
+    { icon_get_selected, "ICON_GET_SELECTED" },
+    { icon_get_subscribed, "ICON_GET_SUBSCRIBED" },
+    { icon_read_group, "ICON_READ_GROUP" },
+    { icon_read_more, "ICON_READ_MORE" },
+    { icon_read_less, "ICON_READ_LESS" },
+    { icon_read_unread_article, "ICON_READ_UNREAD_ARTICLE" },
+    { icon_read_unread_thread, "ICON_READ_UNREAD_THREAD" },
+    { icon_score, "ICON_SCORE" },
+    { icon_search_pulldown, "ICON_SEARCH_PULLDOWN" },
+    { icon_red_flag, "ICON_FLAGGED"},
+    { icon_get_flagged, "ICON_GET_FLAGGED" }
   };
 
   void
@@ -81,10 +92,6 @@ namespace
 
       GtkIconSet * icon_set = gtk_icon_set_new_from_pixbuf (pixbuf);
       gtk_icon_factory_add (factory, my_builtin_icons[i].name, icon_set);
-      g_object_unref (pixbuf);
-
-      //std::cerr << "registered icon " << my_builtin_icons[i].name << std::endl;
-
       g_object_unref (pixbuf);
       gtk_icon_set_unref (icon_set);
     }
@@ -144,9 +151,16 @@ namespace
   void do_read_previous_thread         (GtkAction*) { pan_ui->do_read_previous_thread(); }
   void do_read_parent_article          (GtkAction*) { pan_ui->do_read_parent_article(); }
   void do_show_servers_dialog          (GtkAction*) { pan_ui->do_show_servers_dialog(); }
+  void do_show_sec_dialog              (GtkAction*) { pan_ui->do_show_sec_dialog(); }
   void do_plonk                        (GtkAction*) { pan_ui->do_plonk(); }
   void do_ignore                       (GtkAction*) { pan_ui->do_ignore(); }
   void do_watch                        (GtkAction*) { pan_ui->do_watch(); }
+  void do_flag                         (GtkAction*) { pan_ui->do_flag(); }
+  void do_flag_off                     (GtkAction*) { pan_ui->do_flag_off(); }
+  void do_next_flag                    (GtkAction*) { pan_ui->do_next_flag(); }
+  void do_last_flag                    (GtkAction*) { pan_ui->do_last_flag(); }
+  void do_invert_selection             (GtkAction*) { pan_ui->do_invert_selection(); }
+  void do_mark_all_flagged             (GtkAction*) { pan_ui->do_mark_all_flagged(); }
   void do_show_score_dialog            (GtkAction*) { pan_ui->do_show_score_dialog(); }
   void do_show_new_score_dialog        (GtkAction*) { pan_ui->do_show_new_score_dialog(); }
   void do_cancel_article               (GtkAction*) { pan_ui->do_cancel_article(); }
@@ -155,6 +169,8 @@ namespace
   void do_clear_article_cache          (GtkAction*) { pan_ui->do_clear_article_cache(); }
   void do_mark_article_read            (GtkAction*) { pan_ui->do_mark_article_read(); }
   void do_mark_article_unread          (GtkAction*) { pan_ui->do_mark_article_unread(); }
+  void do_mark_thread_read             (GtkAction*) { pan_ui->do_mark_thread_read(); }
+  void do_mark_thread_unread           (GtkAction*) { pan_ui->do_mark_thread_unread(); }
   void do_post                         (GtkAction*) { pan_ui->do_post(); }
   void do_followup_to                  (GtkAction*) { pan_ui->do_followup_to(); }
   void do_reply_to                     (GtkAction*) { pan_ui->do_reply_to(); }
@@ -230,6 +246,12 @@ namespace
     set_new_match_on_score_state (new_state);
   }
 
+  void do_toggle_rules  (GtkToggleAction * a)
+  {
+    prefs->_rules_enabled = gtk_toggle_action_get_active (a);
+    pan_ui->do_enable_toggle_rules(prefs->_rules_enabled);
+  }
+
   void do_match_only_watched_articles (GtkToggleAction * a)   { set_new_match_on_score_state (gtk_toggle_action_get_active(a) ? MATCH_WATCHED : prev_score_state); }
   void do_match_watched_articles (GtkToggleAction * a)        { set_score_state_bit_from_toggle (a, MATCH_WATCHED); }
   void do_match_high_scoring_articles (GtkToggleAction * a)   { set_score_state_bit_from_toggle (a, MATCH_HIGH_SCORING); }
@@ -247,362 +269,408 @@ namespace
   void do_match_only_my_articles (GtkToggleAction * a) { pan_ui->do_match_only_my_articles (gtk_toggle_action_get_active(a)); }
   void do_match_only_unread_articles (GtkToggleAction * a) { pan_ui->do_match_only_unread_articles (gtk_toggle_action_get_active(a)); }
 
-  GtkActionEntry entries[] =
-  {
-    { "file-menu", NULL, N_("_File"), NULL, NULL, NULL },
-    { "edit-menu", NULL, N_("_Edit"), NULL, NULL, NULL },
-    { "view-layout-menu", NULL, N_("_Layout"), NULL, NULL, NULL },
-    { "view-group-pane-menu", NULL, N_("_Group Pane"), NULL, NULL, NULL },
-    { "view-header-pane-menu", NULL, N_("_Header Pane"), NULL, NULL, NULL },
-    { "view-body-pane-menu", NULL, N_("_Body Pane"), NULL, NULL, NULL },
-    { "view-menu", NULL, N_("_View"), NULL, NULL, NULL },
-    { "filter-menu", NULL, N_("Filte_r"), NULL, NULL, NULL },
-    { "go-menu", NULL, N_("_Go"), NULL, NULL, NULL },
-    { "actions-menu", NULL, N_("_Actions"), NULL, NULL, NULL },
-    { "article-actions-menu", NULL, N_("_Articles"), NULL, NULL, NULL },
-    { "group-actions-menu", NULL, N_("G_roups"), NULL, NULL, NULL },
-    { "posting-actions-menu", NULL, N_("_Post"), NULL, NULL, NULL },
-    { "post-menu", NULL, N_("_Post"), NULL, NULL, NULL },
-    { "help-menu", NULL, N_("_Help"), NULL, NULL, NULL },
-
-    { "set-charset", NULL,
-      N_("Set Character _Encoding..."), NULL,
-      N_("Set Character Encoding..."),
-      G_CALLBACK(do_prompt_for_charset) },
-
-    { "read-selected-group", "ICON_READ_MORE",
-      N_("_Read Group"), NULL,
-      N_("Read Group"),
-      G_CALLBACK(do_read_selected_group) },
-
-    { "mark-groups-read", GTK_STOCK_CLEAR,
-      N_("_Mark Selected Groups Read"), "<control><shift>M",
-      N_("Mark Selected Groups Read"),
-      G_CALLBACK(do_mark_selected_groups_read) },
-
-    { "delete-groups-articles", GTK_STOCK_DELETE,
-      N_("_Delete Selected Groups' Articles"), "<control><shift>Delete",
-      N_("Delete Selected Groups' Articles"),
-      G_CALLBACK(do_clear_selected_groups) },
-
-    { "get-new-headers-in-selected-groups", "ICON_GET_SELECTED",
-      N_("Get New _Headers in Selected Groups"), "A",
-      N_("Get New Headers in Selected Groups"),
-      G_CALLBACK(do_xover_selected_groups) },
-
-    { "get-new-headers-in-subscribed-groups", "ICON_GET_SUBSCRIBED",
-      N_("Get New _Headers in Subscribed Groups"), "<shift>A",
-      N_("Get New Headers in Subscribed Groups"),
-      G_CALLBACK(do_xover_subscribed_groups) },
-
-    { "download-headers", "ICON_GET_DIALOG",
-      N_("Get _Headers..."), "",
-      N_("Get Headers..."),
-      G_CALLBACK(do_download_headers) },
-
-    { "refresh-group-list", NULL,
-      N_("Refresh Group List"), "",
-      N_("Refresh Group List"),
-      G_CALLBACK(do_refresh_groups) },
-
-    { "subscribe", NULL,
-      N_("_Subscribe"), NULL,
-      N_("Subscribe"),
-      G_CALLBACK(do_subscribe_selected_groups) },
-
-    { "unsubscribe", NULL,
-      N_("_Unsubscribe"), NULL,
-      N_("Unsubscribe"),
-      G_CALLBACK(do_unsubscribe_selected_groups) },
-
-    { "save-articles", GTK_STOCK_SAVE,
-      N_("_Save Articles..."), "<shift>S",
-      N_("Save Articles..."),
-      G_CALLBACK(do_save_articles) },
-
-    { "save-articles-from-nzb", GTK_STOCK_SAVE,
-      N_("Save Articles from this _NZB..."), "<control><shift>S",
-      N_("Save Articles from this NZB"),
-      G_CALLBACK(do_save_articles_from_nzb) },
-
-    { "save-articles-to-nzb", GTK_STOCK_SAVE,
-      N_("Save Articles to an NZB _File..."), "<control><shift>F",
-      N_("Save Articles to an NZB File"),
-      G_CALLBACK(do_save_articles_to_nzb) },
-
-    { "print", GTK_STOCK_PRINT,
-      NULL, "<control>P",
-      NULL,
-      G_CALLBACK(do_print) },
-
-    { "import-tasks", GTK_STOCK_OPEN,
-      N_("_Import NZB Files..."), NULL,
-      NULL,
-      G_CALLBACK(do_import_tasks) },
-
-    { "cancel-last-task", GTK_STOCK_CANCEL,
-      N_("_Cancel Last Task"), "<control>Delete",
-      NULL,
-      G_CALLBACK(do_cancel_latest_task) },
-
-    { "show-task-window", NULL,
-      N_("_Task Manager"), NULL,
-      NULL,
-      G_CALLBACK(do_show_task_window) },
-
-    { "show-log-dialog", GTK_STOCK_DIALOG_INFO,
-      N_("_Event Log"), NULL,
-      NULL,
-      G_CALLBACK(do_show_log_window) },
-
-    { "quit", NULL,
-      N_("_Quit"), "<control>Q",
-      NULL,
-      G_CALLBACK(do_quit) },
-
-    { "select-all-articles", NULL,
-      N_("Select _All Articles"), "<control>D",
-      NULL,
-      G_CALLBACK(do_select_all_articles) },
-
-    { "unselect-all-articles", NULL,
-      N_("_Deselect All Articles"), "<shift><control>D",
-      NULL,
-      G_CALLBACK(do_unselect_all_articles) },
-
-    { "add-subthreads-to-selection", NULL,
-      N_("Add Su_bthreads to Selection"), "D",
-      NULL,
-      G_CALLBACK(do_add_subthreads_to_selection) },
-
-    { "add-threads-to-selection", NULL,
-      N_("Add _Threads to Selection"), "<shift>D",
-      NULL,
-      G_CALLBACK(do_add_threads_to_selection) },
-
-    { "add-similar-articles-to-selection", NULL,
-      N_("Add _Similar Articles to Selection"), "<control>S",
-      NULL,
-      G_CALLBACK(do_add_similar_to_selection) },
-
-    { "select-article-body", NULL,
-      N_("Select Article _Body"), NULL,
-      NULL,
-      G_CALLBACK(do_select_article_body) },
-
-    { "show-preferences-dialog", GTK_STOCK_PREFERENCES,
-      N_("Edit _Preferences"), NULL,
-      NULL,
-      G_CALLBACK(do_show_preferences_dialog) },
-
-    { "show-group-preferences-dialog", GTK_STOCK_PREFERENCES,
-      N_("Edit Selected _Group's Preferences"), NULL,
-      NULL,
-      G_CALLBACK(do_show_group_preferences_dialog) },
-
-    { "show-profiles-dialog", GTK_STOCK_EDIT,
-      N_("Edit P_osting Profiles"), NULL,
-      NULL,
-      G_CALLBACK(do_show_profiles_dialog) },
-
-    { "show-servers-dialog", GTK_STOCK_NETWORK,
-      N_("Edit _News Servers"), NULL,
-      NULL,
-      G_CALLBACK(do_show_servers_dialog) },
-
-    { "jump-to-group-tab", GTK_STOCK_JUMP_TO,
-      N_("Jump to _Group Tab"), "1",
-      NULL,
-      G_CALLBACK(do_jump_to_group_tab) },
-
-    { "jump-to-header-tab", GTK_STOCK_JUMP_TO,
-      N_("Jump to _Header Tab"), "2",
-      NULL,
-      G_CALLBACK(do_jump_to_header_tab) },
-
-    { "jump-to-body-tab", GTK_STOCK_JUMP_TO,
-      N_("Jump to _Body Tab"), "3",
-      NULL,
-      G_CALLBACK(do_jump_to_body_tab) },
-
-    { "rot13-selected-text", NULL,
-      N_("_Rot13 Selected Text"), "<control><shift>R",
-      NULL,
-      G_CALLBACK(do_rot13_selected_text) },
-
-    { "clear-header-pane", NULL,
-      N_("Clear _Header Pane"), NULL,
-      NULL,
-      G_CALLBACK(do_clear_header_pane) },
-    { "clear-body-pane", NULL,
-      N_("Clear _Body Pane"), NULL, 
-      NULL,
-      G_CALLBACK(do_clear_body_pane) },
-
-    { "download-selected-article", "ICON_DISK",
-      N_("Cache Article"), NULL, 
-      NULL,
-      G_CALLBACK(do_download_selected_article) },
-
-    { "read-selected-article", "ICON_READ_MORE",
-      N_("Read Article"), NULL, 
-      NULL,
-      G_CALLBACK(do_read_selected_article) },
-
-    { "show-selected-article-info", NULL,
-      N_("Show Article Information"), NULL, 
-      NULL,
-      G_CALLBACK(do_show_selected_article_info) },
-
-    { "read-more", "ICON_READ_MORE",
-      N_("Read _More"), "space",
-      N_("Read More"),
-      G_CALLBACK(do_read_more) },
-
-    { "read-less", "ICON_READ_LESS",
-      N_("Read _Back"), "BackSpace",
-      N_("Read Back"),
-      G_CALLBACK(do_read_less) },
-
-    { "read-next-unread-group", "ICON_READ_UNREAD_GROUP",
-      N_("Next _Unread Group"), "G", 
-      NULL,
-      G_CALLBACK(do_read_next_unread_group) },
-
-    { "read-next-group", "ICON_READ_GROUP",
-      N_("Next _Group"), "<shift>G", 
-      NULL,
-      G_CALLBACK(do_read_next_group) },
-
-    { "read-next-unread-article", "ICON_READ_UNREAD_ARTICLE",
-      N_("Next _Unread Article"), "N",
-      N_("Next Unread Article"),
-      G_CALLBACK(do_read_next_unread_article) },
-
-    { "read-next-article", NULL,
-      N_("Next _Article"), "<control>N",
-      N_("Next Article"),
-      G_CALLBACK(do_read_next_article) },
-
-    { "read-next-watched-article", NULL,
-      N_("Next _Watched Article"), "<control><shift>N", 
-      NULL,
-      G_CALLBACK(do_read_next_watched_article) },
-
-    { "read-next-unread-thread", "ICON_READ_UNREAD_THREAD",
-      N_("Next Unread _Thread"), "T",
-      N_("Next Unread Thread"),
-      G_CALLBACK(do_read_next_unread_thread) },
-
-    { "read-next-thread", NULL,
-      N_("Next Threa_d"), "<control>T", 
-      NULL,
-      G_CALLBACK(do_read_next_thread) },
-
-    { "read-previous-article", NULL,
-      N_("Pre_vious Article"), "V", 
-      NULL,
-      G_CALLBACK(do_read_previous_article) },
-
-    { "read-previous-thread", NULL,
-      N_("Previous _Thread"), "<control>V", 
-      NULL,
-
-      G_CALLBACK(do_read_previous_thread) },
-    { "read-parent-article", NULL,
-      N_("_Parent Article"), "U", 
-      NULL,
-      G_CALLBACK(do_read_parent_article) },
-
-    { "plonk", NULL,
-      N_("Ignore _Author"), NULL, 
-      NULL,
-      G_CALLBACK(do_plonk) },
-    { "watch-thread", NULL,
-      N_("_Watch Thread"), NULL, 
-      NULL,
-      G_CALLBACK(do_watch) },
-
-    { "ignore-thread", NULL,
-      N_("_Ignore Thread"), NULL, 
-      NULL,
-      G_CALLBACK(do_ignore) },
-
-    { "view-article-score", "ICON_SCORE",
-      N_("Edit Article's Watch/Ignore/Score..."), "<control><shift>C", 
-      NULL,
-      G_CALLBACK(do_show_score_dialog) },
-
-    { "add-article-score", "ICON_SCORE",
-      N_("Add a _Scoring Rule..."), "S", 
-      NULL,
-      G_CALLBACK(do_show_new_score_dialog) },
-
-    { "cancel-article", NULL,
-      N_("Cance_l Article..."), NULL, 
-      NULL,
-      G_CALLBACK(do_cancel_article) },
-
-    { "supersede-article", NULL,
-      N_("_Supersede Article..."), NULL, 
-      NULL,
-      G_CALLBACK(do_supersede_article) },
-
-    { "delete-article", GTK_STOCK_DELETE,
-      N_("_Delete Article"), "Delete", 
-      NULL,
-      G_CALLBACK(do_delete_article) },
-
-    { "clear-article-cache", NULL,
-      N_("Clear Article Cache"), NULL, 
-      NULL,
-      G_CALLBACK(do_clear_article_cache) },
-
-    { "mark-article-read", "ICON_ARTICLE_READ",
-      N_("_Mark Article as Read"), "M", 
-      NULL,
-      G_CALLBACK(do_mark_article_read) },
-
-    { "mark-article-unread", "ICON_ARTICLE_UNREAD",
-      N_("Mark Article as _Unread"), "<control>M", 
-      NULL,
-      G_CALLBACK(do_mark_article_unread) },
-
-    { "post", "ICON_COMPOSE_POST",
-      N_("_Post to Newsgroup"), "P",
-      N_("Post to Newsgroup"),
-      G_CALLBACK(do_post) },
-
-    { "followup-to", "ICON_COMPOSE_FOLLOWUP",
-      N_("_Followup to Newsgroup"), "F",
-      N_("Followup to Newsgroup"),
-      G_CALLBACK(do_followup_to) },
-
-    { "reply-to", NULL,
-      N_("_Reply to Author in Mail"), "R", 
-      NULL,
-      G_CALLBACK(do_reply_to) },
-
-    { "pan-web-page", NULL,
-      N_("_Pan Home Page"), NULL, 
-      NULL,
-      G_CALLBACK(do_pan_web) },
-
-    { "bug-report", NULL,
-      N_("Give _Feedback or Report a Bug..."), NULL, 
-      NULL,
-      G_CALLBACK(do_bug_report) },
-
-    { "tip-jar", NULL,
-      N_("_Tip Jar..."), NULL, 
-      NULL,
-      G_CALLBACK(do_tip_jar) },
-
-    { "about-pan", GTK_STOCK_ABOUT,
-      N_("_About"), NULL, 
-      NULL,
-      G_CALLBACK(do_about_pan) }
-  };
+    GtkActionEntry entries[] =
+    {
+      { "file-menu", NULL, N_("_File"), NULL, NULL, NULL },
+      { "edit-menu", NULL, N_("_Edit"), NULL, NULL, NULL },
+      { "view-layout-menu", NULL, N_("_Layout"), NULL, NULL, NULL },
+      { "view-group-pane-menu", NULL, N_("_Group Pane"), NULL, NULL, NULL },
+      { "view-header-pane-menu", NULL, N_("_Header Pane"), NULL, NULL, NULL },
+      { "view-body-pane-menu", NULL, N_("_Body Pane"), NULL, NULL, NULL },
+      { "view-menu", NULL, N_("_View"), NULL, NULL, NULL },
+      { "filter-menu", NULL, N_("Filte_r"), NULL, NULL, NULL },
+      { "go-menu", NULL, N_("_Go"), NULL, NULL, NULL },
+      { "actions-menu", NULL, N_("_Actions"), NULL, NULL, NULL },
+      { "article-actions-menu", NULL, N_("_Articles"), NULL, NULL, NULL },
+      { "group-actions-menu", NULL, N_("G_roups"), NULL, NULL, NULL },
+      { "posting-actions-menu", NULL, N_("_Post"), NULL, NULL, NULL },
+      { "post-menu", NULL, N_("_Post"), NULL, NULL, NULL },
+      { "help-menu", NULL, N_("_Help"), NULL, NULL, NULL },
+
+      { "read-selected-group", "ICON_READ_MORE",
+        N_("_Read Group"), NULL,
+        N_("Read Group"),
+        G_CALLBACK(do_read_selected_group) },
+
+      { "mark-groups-read", GTK_STOCK_CLEAR,
+        N_("_Mark Selected Groups Read"), "<control><shift>M",
+        N_("Mark Selected Groups Read"),
+        G_CALLBACK(do_mark_selected_groups_read) },
+
+      { "delete-groups-articles", GTK_STOCK_DELETE,
+        N_("_Delete Selected Groups' Articles"), "<control><shift>Delete",
+        N_("Delete Selected Groups' Articles"),
+        G_CALLBACK(do_clear_selected_groups) },
+
+      { "get-new-headers-in-selected-groups", "ICON_GET_SELECTED",
+        N_("Get New _Headers in Selected Groups"), "A",
+        N_("Get New Headers in Selected Groups"),
+        G_CALLBACK(do_xover_selected_groups) },
+
+      { "get-new-headers-in-subscribed-groups", "ICON_GET_SUBSCRIBED",
+        N_("Get New _Headers in Subscribed Groups"), "<shift>A",
+        N_("Get New Headers in Subscribed Groups"),
+        G_CALLBACK(do_xover_subscribed_groups) },
+
+      { "update-stats-in-selected-groups", "ICON_GET_SUBSCRIBED",
+        N_("Get New _Headers in Subscribed Groups"), "<shift>A",
+        N_("Get New Headers in Subscribed Groups"),
+        G_CALLBACK(do_xover_subscribed_groups) },
+
+      { "download-headers", "ICON_GET_DIALOG",
+        N_("Get _Headers..."), "",
+        N_("Get Headers..."),
+        G_CALLBACK(do_download_headers) },
+
+      { "refresh-group-list", NULL,
+        N_("Refresh Group List"), "",
+        N_("Refresh Group List"),
+        G_CALLBACK(do_refresh_groups) },
+
+      { "subscribe", NULL,
+        N_("_Subscribe"), NULL,
+        N_("Subscribe"),
+        G_CALLBACK(do_subscribe_selected_groups) },
+
+      { "unsubscribe", NULL,
+        N_("_Unsubscribe"), NULL,
+        N_("Unsubscribe"),
+        G_CALLBACK(do_unsubscribe_selected_groups) },
+
+      { "save-articles", GTK_STOCK_SAVE,
+        N_("_Save Articles..."), "<shift>S",
+        N_("Save Articles..."),
+        G_CALLBACK(do_save_articles) },
+
+      { "save-articles-from-nzb", GTK_STOCK_SAVE,
+        N_("Save Articles from Selected _NZB..."), "<control><shift>S",
+        N_("Save Articles from Selected NZB"),
+        G_CALLBACK(do_save_articles_from_nzb) },
+
+      { "save-articles-to-nzb", GTK_STOCK_SAVE,
+        N_("Save Articles to an NZB _File..."), "<control><shift>F",
+        N_("Save Articles to an NZB File"),
+        G_CALLBACK(do_save_articles_to_nzb) },
+
+      { "print", GTK_STOCK_PRINT,
+        N_("Print"), "<control>P",
+        N_("Print"),
+        G_CALLBACK(do_print) },
+
+      { "import-tasks", GTK_STOCK_OPEN,
+        N_("_Import NZB Files..."), "<constrol>O",
+        NULL,
+        G_CALLBACK(do_import_tasks) },
+
+      { "cancel-last-task", GTK_STOCK_CANCEL,
+        N_("_Cancel Last Task"), "<control>Delete",
+        NULL,
+        G_CALLBACK(do_cancel_latest_task) },
+
+      { "show-task-window", NULL,
+        N_("_Task Manager"), NULL,
+        NULL,
+        G_CALLBACK(do_show_task_window) },
+
+      { "show-log-dialog", GTK_STOCK_DIALOG_INFO,
+        N_("_Event Log"), NULL,
+        NULL,
+        G_CALLBACK(do_show_log_window) },
+
+      { "quit", NULL,
+        N_("_Quit"), "<control>Q",
+        NULL,
+        G_CALLBACK(do_quit) },
+
+      { "select-all-articles", NULL,
+        N_("Select _All Articles"), "<control>D",
+        NULL,
+        G_CALLBACK(do_select_all_articles) },
+
+      { "unselect-all-articles", NULL,
+        N_("_Deselect All Articles"), "<shift><control>D",
+        NULL,
+        G_CALLBACK(do_unselect_all_articles) },
+
+      { "add-subthreads-to-selection", NULL,
+        N_("Add Su_bthreads to Selection"), "D",
+        NULL,
+        G_CALLBACK(do_add_subthreads_to_selection) },
+
+      { "add-threads-to-selection", NULL,
+        N_("Add _Threads to Selection"), "<shift>D",
+        NULL,
+        G_CALLBACK(do_add_threads_to_selection) },
+
+      { "add-similar-articles-to-selection", NULL,
+        N_("Add _Similar Articles to Selection"), "<control>S",
+        NULL,
+        G_CALLBACK(do_add_similar_to_selection) },
+
+      { "select-article-body", NULL,
+        N_("Select Article _Body"), NULL,
+        NULL,
+        G_CALLBACK(do_select_article_body) },
+
+      { "show-preferences-dialog", GTK_STOCK_PREFERENCES,
+        N_("Edit _Preferences"), NULL,
+        NULL,
+        G_CALLBACK(do_show_preferences_dialog) },
+
+      { "show-group-preferences-dialog", GTK_STOCK_PREFERENCES,
+        N_("Edit Selected _Group's Preferences"), NULL,
+        NULL,
+        G_CALLBACK(do_show_group_preferences_dialog) },
+
+      { "show-profiles-dialog", GTK_STOCK_EDIT,
+        N_("Edit P_osting Profiles"), NULL,
+        NULL,
+        G_CALLBACK(do_show_profiles_dialog) },
+
+      { "show-servers-dialog", GTK_STOCK_NETWORK,
+        N_("Edit _News Servers"), NULL,
+        NULL,
+        G_CALLBACK(do_show_servers_dialog) },
+
+      { "jump-to-group-tab", GTK_STOCK_JUMP_TO,
+        N_("Jump to _Group Tab"), "1",
+        NULL,
+        G_CALLBACK(do_jump_to_group_tab) },
+
+      { "jump-to-header-tab", GTK_STOCK_JUMP_TO,
+        N_("Jump to _Header Tab"), "2",
+        NULL,
+        G_CALLBACK(do_jump_to_header_tab) },
+
+      { "jump-to-body-tab", GTK_STOCK_JUMP_TO,
+        N_("Jump to _Body Tab"), "3",
+        NULL,
+        G_CALLBACK(do_jump_to_body_tab) },
+
+      { "rot13-selected-text", NULL,
+        N_("_Rot13 Selected Text"), "<control><shift>R",
+        NULL,
+        G_CALLBACK(do_rot13_selected_text) },
+
+      { "clear-header-pane", NULL,
+        N_("Clear _Header Pane"), NULL,
+        NULL,
+        G_CALLBACK(do_clear_header_pane) },
+
+      { "clear-body-pane", NULL,
+        N_("Clear _Body Pane"), NULL,
+        NULL,
+        G_CALLBACK(do_clear_body_pane) },
+
+      { "download-selected-article", "ICON_DISK",
+        N_("Cache Article"), NULL,
+        NULL,
+        G_CALLBACK(do_download_selected_article) },
+
+      { "read-selected-article", "ICON_READ_MORE",
+        N_("Read Article"), NULL,
+        NULL,
+        G_CALLBACK(do_read_selected_article) },
+
+      { "show-selected-article-info", NULL,
+        N_("Show Article Information"), NULL,
+        NULL,
+        G_CALLBACK(do_show_selected_article_info) },
+
+      { "read-more", "ICON_READ_MORE",
+        N_("Read _More"), "space",
+        N_("Read More"),
+        G_CALLBACK(do_read_more) },
+
+      { "read-less", "ICON_READ_LESS",
+        N_("Read _Back"), "BackSpace",
+        N_("Read Back"),
+        G_CALLBACK(do_read_less) },
+
+      { "read-next-unread-group", "ICON_READ_UNREAD_GROUP",
+        N_("Next _Unread Group"), "G",
+        NULL,
+        G_CALLBACK(do_read_next_unread_group) },
+
+      { "read-next-group", "ICON_READ_GROUP",
+        N_("Next _Group"), "<shift>G",
+        NULL,
+        G_CALLBACK(do_read_next_group) },
+
+      { "read-next-unread-article", "ICON_READ_UNREAD_ARTICLE",
+        N_("Next _Unread Article"), "N",
+        N_("Next Unread Article"),
+        G_CALLBACK(do_read_next_unread_article) },
+
+      { "read-next-article", NULL,
+        N_("Next _Article"), "<control>N",
+        N_("Next Article"),
+        G_CALLBACK(do_read_next_article) },
+
+      { "read-next-watched-article", NULL,
+        N_("Next _Watched Article"), "<control><shift>N",
+        NULL,
+        G_CALLBACK(do_read_next_watched_article) },
+
+      { "read-next-unread-thread", "ICON_READ_UNREAD_THREAD",
+        N_("Next Unread _Thread"), "T",
+        N_("Next Unread Thread"),
+        G_CALLBACK(do_read_next_unread_thread) },
+
+      { "read-next-thread", NULL,
+        N_("Next Threa_d"), "<control>T",
+        NULL,
+        G_CALLBACK(do_read_next_thread) },
+
+      { "read-previous-article", NULL,
+        N_("Pre_vious Article"), "V",
+        NULL,
+        G_CALLBACK(do_read_previous_article) },
+
+      { "read-previous-thread", NULL,
+        N_("Previous _Thread"), "<control>V",
+        NULL,
+
+        G_CALLBACK(do_read_previous_thread) },
+      { "read-parent-article", NULL,
+        N_("_Parent Article"), "U",
+        NULL,
+        G_CALLBACK(do_read_parent_article) },
+
+      { "plonk", NULL,
+        N_("Ignore _Author"), NULL,
+        NULL,
+        G_CALLBACK(do_plonk) },
+      { "watch-thread", NULL,
+        N_("_Watch Thread"), NULL,
+        NULL,
+        G_CALLBACK(do_watch) },
+
+      { "ignore-thread", NULL,
+        N_("_Ignore Thread"), NULL,
+        NULL,
+        G_CALLBACK(do_ignore) },
+
+      { "flag-thread", "ICON_FLAGGED",
+        N_("_Toggle Flag On/Off for Thread"), "X",
+        N_("_Toggle Flag On/Off for Thread"),
+        G_CALLBACK(do_flag) },
+
+      { "unflag-thread", NULL,
+        N_("_Turn Flag Off for Thread"), "<shift>X",
+        N_("_Turn Flag Off for Thread"),
+        G_CALLBACK(do_flag_off) },
+
+      { "select-all-flagged", NULL,
+        N_("_Select All Flagged Threads"), "<control>X",
+        N_("_Select All Flagged Threads"),
+        G_CALLBACK(do_mark_all_flagged) },
+
+      { "next-flagged", NULL,
+        N_("_Go to Next Flagged Thread"), "plus",
+        N_("_Go to Next Flagged Thread"),
+        G_CALLBACK(do_next_flag) },
+
+      { "last-flagged", NULL,
+        N_("_Go to Last Flagged Thread"), "minus",
+        N_("_Go to Last Flagged Thread"),
+        G_CALLBACK(do_last_flag) },
+
+      { "invert-selection", NULL,
+        N_("_Invert Selection"), "<control>I",
+        N_("_Invert Selection"),
+        G_CALLBACK(do_invert_selection) },
+
+      { "view-article-score", "ICON_SCORE",
+        N_("Edit Article's Watch/Ignore/Score..."), "<control><shift>C",
+        NULL,
+        G_CALLBACK(do_show_score_dialog) },
+
+      { "add-article-score", "ICON_SCORE",
+        N_("Add a _Scoring Rule..."), "S",
+        NULL,
+        G_CALLBACK(do_show_new_score_dialog) },
+
+      { "cancel-article", NULL,
+        N_("Cance_l Article..."), NULL,
+        NULL,
+        G_CALLBACK(do_cancel_article) },
+
+      { "supersede-article", NULL,
+        N_("_Supersede Article..."), NULL,
+        NULL,
+        G_CALLBACK(do_supersede_article) },
+
+      { "delete-article", GTK_STOCK_DELETE,
+        N_("_Delete Article"), "Delete",
+        NULL,
+        G_CALLBACK(do_delete_article) },
+
+      { "clear-article-cache", NULL,
+        N_("Clear Article Cache"), NULL,
+        NULL,
+        G_CALLBACK(do_clear_article_cache) },
+
+      { "mark-article-read", "ICON_ARTICLE_READ",
+        N_("_Mark Article as Read"), "M",
+        NULL,
+        G_CALLBACK(do_mark_article_read) },
+
+      { "mark-article-unread", "ICON_ARTICLE_UNREAD",
+        N_("Mark Article as _Unread"), "<control>M",
+        NULL,
+        G_CALLBACK(do_mark_article_unread) },
+
+      ///TODO create icons for the next two actions
+      { "mark-thread-read", "ICON_ARTICLE_READ",
+        N_("_Mark Thread as Read"), "<Shift>T",
+        NULL,
+        G_CALLBACK(do_mark_thread_read) },
+
+      { "mark-thread-unread", "ICON_ARTICLE_UNREAD",
+        N_("Mark Thread as _Unread"), "<control><Shift>T",
+        NULL,
+        G_CALLBACK(do_mark_thread_unread) },
+
+      { "post", "ICON_COMPOSE_POST",
+        N_("_Post to Newsgroup"), "P",
+        N_("Post to Newsgroup"),
+        G_CALLBACK(do_post) },
+
+      { "followup-to", "ICON_COMPOSE_FOLLOWUP",
+        N_("_Followup to Newsgroup"), "F",
+        N_("Followup to Newsgroup"),
+        G_CALLBACK(do_followup_to) },
+
+      { "reply-to", NULL,
+        N_("_Reply to Author in Mail"), "R",
+        NULL,
+        G_CALLBACK(do_reply_to) },
+
+      { "pan-web-page", NULL,
+        N_("_Pan Home Page"), NULL,
+        NULL,
+        G_CALLBACK(do_pan_web) },
+
+      { "bug-report", NULL,
+        N_("Give _Feedback or Report a Bug..."), NULL,
+        NULL,
+        G_CALLBACK(do_bug_report) },
+
+      { "tip-jar", NULL,
+        N_("_Tip Jar..."), NULL,
+        NULL,
+        G_CALLBACK(do_tip_jar) },
+
+      { "about-pan", GTK_STOCK_ABOUT,
+        N_("_About"), NULL,
+        NULL,
+        G_CALLBACK(do_about_pan) },
+
+      { "show-sec-dialog", GTK_STOCK_DIALOG_AUTHENTICATION,
+        N_("Edit _SSL Certificates"), NULL,
+        NULL,
+        G_CALLBACK(do_show_sec_dialog) } };
 
   void prefs_toggle_callback_impl (GtkToggleAction * action)
   {
@@ -615,7 +683,7 @@ namespace
   GtkToggleActionEntry toggle_entries[] =
   {
     { "thread-headers",           NULL, N_("_Thread Headers"),                NULL, NULL, G_CALLBACK(prefs_toggle_callback_impl), true },
-    { "wrap-article-body",        NULL, N_("_Wrap Article Body"),              "W", NULL, G_CALLBACK(prefs_toggle_callback_impl), false },
+    { "wrap-article-body",GTK_STOCK_JUSTIFY_FILL, N_("_Wrap Article Body"),    "W", NULL, G_CALLBACK(prefs_toggle_callback_impl), false },
     { "mute-quoted-text",         NULL, N_("Mute _Quoted Text"),               "Q", NULL, G_CALLBACK(prefs_toggle_callback_impl), true },
     { "show-all-headers",         NULL, N_("Show All _Headers in Body Pane"),  "H", NULL, G_CALLBACK(prefs_toggle_callback_impl), false },
     { "show-smilies-as-graphics", NULL, N_("Show _Smilies as Graphics"),      NULL, NULL, G_CALLBACK(prefs_toggle_callback_impl), true },
@@ -645,7 +713,9 @@ namespace
     { "match-medium-scoring-articles", NULL, N_("Match Scores of 1...4999 (Me_dium)"), NULL, NULL, G_CALLBACK(do_match_medium_scoring_articles), true },
     { "match-normal-scoring-articles", NULL, N_("Match Scores of 0 (_Normal)"), NULL, NULL, G_CALLBACK(do_match_normal_scoring_articles), true },
     { "match-low-scoring-articles", NULL, N_("Match Scores of -9998...-1 (_Low)"), NULL, NULL, G_CALLBACK(do_match_low_scoring_articles), true },
-    { "match-ignored-articles", NULL, N_("Match Scores of -9999 (_Ignored)"), NULL, NULL, G_CALLBACK(do_match_ignored_articles), false }
+    { "match-ignored-articles", NULL, N_("Match Scores of -9999 (_Ignored)"), NULL, NULL, G_CALLBACK(do_match_ignored_articles), false },
+
+    { "enable-rules", NULL, N_("Enable/Disable All _Rules"), "R", NULL, G_CALLBACK(do_toggle_rules), true    }
   };
 
   const guint n_toggle_entries (G_N_ELEMENTS(toggle_entries));
@@ -696,8 +766,36 @@ namespace
 }
 
 void
-pan :: add_actions (PanUI * ui, GtkUIManager * ui_manager, Prefs * p)
+pan :: add_actions (PanUI * ui, GtkUIManager * ui_manager, Prefs * p, Data* data)
 {
+
+  GtkActionGroup * action_group = _group = gtk_action_group_new ("Actions");
+  gtk_action_group_set_translation_domain (action_group, NULL);
+
+  for (int i=0;i<n_entries;++i)
+  {
+    const gchar* abbrev = entries[i].tooltip ? entries[i].tooltip : entries[i].label ? entries[i].label : "";
+    action_trans[entries[i].name] = abbrev;
+    const gchar* translation = gtk_action_group_translate_string (action_group, action_trans[entries[i].name].c_str());
+    action_trans[entries[i].name] = translation ? translation : "";
+  }
+
+  for (int i=0;i<n_toggle_entries;++i)
+  {
+    const gchar* abbrev = toggle_entries[i].tooltip ? toggle_entries[i].tooltip : toggle_entries[i].label ? toggle_entries[i].label : "";
+    action_trans[toggle_entries[i].name] = abbrev;
+    const gchar* translation = gtk_action_group_translate_string (action_group, action_trans[toggle_entries[i].name].c_str());
+    action_trans[toggle_entries[i].name] = translation ? translation : "";
+  }
+
+  for (int i=0;i<G_N_ELEMENTS(match_toggle_entries);++i)
+  {
+    const gchar* abbrev = match_toggle_entries[i].tooltip ? match_toggle_entries[i].tooltip : match_toggle_entries[i].label ? match_toggle_entries[i].label : "";
+    action_trans[match_toggle_entries[i].name] = abbrev;
+    const gchar* translation = gtk_action_group_translate_string (action_group, action_trans[match_toggle_entries[i].name].c_str());
+    action_trans[match_toggle_entries[i].name] = translation ? translation : "";
+  }
+
   pan_ui = ui;
   prefs = p;
 
@@ -726,8 +824,6 @@ pan :: add_actions (PanUI * ui, GtkUIManager * ui_manager, Prefs * p)
   for (GtkActionEntry *it(entries), *end(it+n_entries); it!=end; ++it)
     ensure_tooltip (it);
 
-  GtkActionGroup * action_group = _group = gtk_action_group_new ("Actions");
-  gtk_action_group_set_translation_domain (action_group, NULL);
   gtk_action_group_add_actions (action_group, entries, n_entries, NULL);
   gtk_action_group_add_toggle_actions (action_group, toggle_entries, n_toggle_entries, NULL);
   gtk_action_group_add_radio_actions (action_group,
