@@ -21,11 +21,10 @@
 #include <cctype>
 #include <cmath>
 #include <iostream>
+#include <sstream>
 extern "C" {
   #include <glib/gi18n.h>
-  #include <gtk/gtk.h>
-  #include <gdk/gdk.h>
-  #include <gdk/gdkkeysyms.h>
+  #include "gtk-compat.h"
   #include <gmime/gmime.h>
 }
 #include <gdk-pixbuf/gdk-pixbuf.h>
@@ -33,6 +32,7 @@ extern "C" {
 #include <pan/general/log.h>
 #include <pan/general/macros.h>
 #include <pan/general/utf8-utils.h>
+#include <pan/general/e-util.h>
 #include <pan/usenet-utils/gnksa.h>
 #include <pan/usenet-utils/mime-utils.h>
 #include <pan/usenet-utils/url-find.h>
@@ -42,12 +42,45 @@ extern "C" {
 #include "tango-colors.h"
 #include "xface.h"
 #include "url.h"
-#include "gtk_compat.h"
+#include "gtk-compat.h"
+#include "save-attach-ui.h"
 
 #define FIRST_PICTURE "first-picture"
 
 using namespace pan;
 
+/***
+****
+***/
+
+namespace
+{
+
+  GtkWindow* get_window (GtkWidget* w)
+  {
+    return GTK_WINDOW (gtk_widget_get_toplevel (w));
+  }
+
+  enum Icons
+  {
+    ICON_SIG_OK,
+    ICON_SIG_FAIL,
+    NUM_ICONS
+  };
+
+  struct Icon {
+  const guint8 * pixbuf_txt;
+  GdkPixbuf * pixbuf;
+  } icons[NUM_ICONS] = {
+    { icon_sig_ok,          0 },
+    { icon_sig_fail,        0 }
+  };
+}
+
+/***
+****
+***/
+
 /**
 ***  Pixbuf Cache
 **/
@@ -100,7 +133,7 @@ namespace
 
   PixbufCache& get_pixbuf_cache (gpointer gp) {
     static const char * PIXBUF_CACHE ("pixbuf-cache");
-    GObject * o (G_OBJECT (gp)); 
+    GObject * o (G_OBJECT (gp));
     PixbufCache *pc ((PixbufCache*) g_object_get_data(o, PIXBUF_CACHE));
     if (!pc) {
       pc = new PixbufCache ();
@@ -337,10 +370,11 @@ namespace
                                const GtkAllocation  * size,
                                GtkTextTag           * apply_tag)
   {
+
     const int begin_offset (gtk_text_iter_get_offset (iter));
 
-    GdkPixbuf * original (0);    
-    GdkPixbuf * old_scaled (0);    
+    GdkPixbuf * original (0);
+    GdkPixbuf * old_scaled (0);
     if (!get_pixbuf_at_offset (buf, begin_offset, original, old_scaled))
       return;
 
@@ -498,6 +532,7 @@ namespace
     gboolean handled (false);
 
     g_return_val_if_fail (GTK_IS_TEXT_VIEW(w), false);
+
     const bool up = event->keyval==GDK_KEY_Up || event->keyval==GDK_KEY_KP_Up;
     const bool down = event->keyval==GDK_KEY_Down || event->keyval==GDK_KEY_KP_Down;
 
@@ -559,7 +594,7 @@ namespace
       for (;;) {
         if (str == line_end)
           break;
-        else if (text_massager->is_quote_character (g_utf8_get_char (str))) 
+        else if (text_massager->is_quote_character (g_utf8_get_char (str)))
           ++depth;
         else if (!g_unichar_isspace(g_utf8_get_char(str)))
           break;
@@ -606,7 +641,7 @@ namespace
   }
 
   enum TagMode { ADD, REPLACE };
- 
+
   void
   set_section_tag (GtkTextBuffer     * buffer,
                    GtkTextIter       * start,
@@ -681,6 +716,7 @@ namespace
                              bool                  do_markup,
                              bool                  do_urls)
   {
+
     g_return_if_fail (buffer!=0);
     g_return_if_fail (GTK_IS_TEXT_BUFFER(buffer));
 
@@ -707,8 +743,8 @@ namespace
     int offset (0);
     if (GNKSA::find_signature_delimiter (v, offset) != GNKSA::SIG_NONE)
       sig_point = v.str + offset;
-   
-    // colorize the quoted text 
+
+    // colorize the quoted text
     GtkTextIter mark_end;
     std::string last_quote_tag;
     bool is_sig (false);
@@ -798,7 +834,7 @@ namespace
         }
       }
     }
-    
+
     // colorize urls
     if (do_urls) {
       StringView area;
@@ -838,7 +874,7 @@ namespace
       gsize bytesLeft = buffer->len;
       guchar * data = buffer->data;
 
-      // ticket #467446 - workaround gdkpixbuf <= 2.12.x's 
+      // ticket #467446 - workaround gdkpixbuf <= 2.12.x's
       // jpg loader bug (#494667) by feeding the loader in
       // smaller chunks
       while( bytesLeft > 0 )
@@ -881,10 +917,11 @@ namespace
     g_object_unref (G_OBJECT(l));
     return pixbuf;
   }
+
 }
 
 void
-BodyPane :: append_part (GMimeObject * obj, GtkAllocation * widget_size)
+BodyPane :: append_part (GMimeObject * parent, GMimeObject * obj, GtkAllocation * widget_size)
 {
   bool is_done (false);
 
@@ -922,6 +959,7 @@ BodyPane :: append_part (GMimeObject * obj, GtkAllocation * widget_size)
       // rembember the location of the first picture.
       if (gtk_text_buffer_get_mark (_buffer, FIRST_PICTURE) == NULL)
         gtk_text_buffer_create_mark (_buffer, FIRST_PICTURE, &iter, true);
+        gtk_text_buffer_create_mark (_buffer, FIRST_PICTURE, &iter, true);
 
       // add the picture
       const int begin_offset (gtk_text_iter_get_offset (&iter));
@@ -965,26 +1003,27 @@ BodyPane :: append_part (GMimeObject * obj, GtkAllocation * widget_size)
     is_done = true;
   }
 
-  // otherwise, bitch and moan.
+  // otherwise, add to list of attachments
   if (!is_done) {
     const char * filename = g_mime_part_get_filename (part);
     char * pch = (filename && *filename)
-      ? g_strdup_printf (_("Attachment not shown: MIME type %s/%s; filename %s\n"), type->type, type->subtype, filename)
-      : g_strdup_printf (_("Attachment not shown: MIME type %s/%s\n"), type->type, type->subtype);
-    GtkTextIter iter;
-    gtk_text_buffer_get_end_iter (_buffer, &iter);
-    gtk_text_buffer_insert (_buffer, &iter, pch, -1);
-    g_free (pch);
+      ? g_strdup_printf ("%s", filename)
+      : g_strdup_printf (_("Unnamed File"));
+
+    add_attachment_to_toolbar (pch);
+
+    _freeme.insert(pch);
   }
 }
+
 void
-BodyPane :: foreach_part_cb (GMimeObject* /*parent*/, GMimeObject* o, gpointer self)
+BodyPane :: foreach_part_cb (GMimeObject* parent, GMimeObject* o, gpointer self)
 {
   BodyPane * pane = static_cast<BodyPane*>(self);
   GtkWidget * w (pane->_text);
   GtkAllocation aloc;
   gtk_widget_get_allocation(w, &aloc);
-  pane->append_part (o, &aloc);
+  pane->append_part (parent, o, &aloc);
 }
 
 
@@ -1026,6 +1065,7 @@ namespace
     s += e;
     const size_t retval (g_utf8_strlen(key,-1) + g_utf8_strlen(utf8_val.c_str(),-1) + 2);
     g_free (e);
+
     return retval;
   }
 
@@ -1038,6 +1078,7 @@ namespace
     const char * val (msg ? g_mime_object_get_header ((GMimeObject *) msg, key) : "");
     return add_header_line (s, key_i18n, key, val, fallback_charset);
   }
+
 }
 
 void
@@ -1045,7 +1086,7 @@ BodyPane :: set_text_from_message (GMimeMessage * message)
 {
   const char * fallback_charset (_charset.empty() ? 0 : _charset.c_str());
 
-  // manditory headers...
+  // mandatory headers...
   std::string s;
   size_t w(0), l(0);
   l = add_header_line (s, message, _("Subject"), "Subject", fallback_charset);
@@ -1079,7 +1120,7 @@ BodyPane :: set_text_from_message (GMimeMessage * message)
       }
     }
   }
-  
+
   s.resize (s.size()-1); // remove trailing linefeed
   gtk_label_set_markup (GTK_LABEL(_headers), s.c_str());
 
@@ -1148,6 +1189,8 @@ BodyPane :: set_text_from_message (GMimeMessage * message)
     g_free (headers);
   }
 
+  clear_attachments();
+
   // FIXME: need to set a mark here so that when user hits follow-up,
   // the all-headers don't get included in the followup
 
@@ -1165,6 +1208,7 @@ BodyPane :: set_text_from_message (GMimeMessage * message)
     gtk_text_buffer_get_start_iter  (_buffer, &iter);
     gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW(_text), &iter, 0.0, true, 0.0, 0.0);
   }
+
 }
 
 void
@@ -1173,6 +1217,65 @@ BodyPane :: refresh ()
   set_text_from_message (_message);
 }
 
+namespace
+{
+
+  /** Separates user id into name and email address */
+  std::pair<std::string,std::string> get_email_address(std::string& s)
+  {
+    std::pair<std::string,std::string> ret;
+    size_t in = s.find("<");
+    size_t out = s.find(">");
+    if (in == std::string::npos ||
+        out == std::string::npos)
+      return ret;
+
+    ret.first = s.substr(0,in-1);
+    ret.second = s.substr (in+1,out-in-1);
+
+    return ret;
+  }
+}
+
+#ifdef HAVE_GMIME_CRYPTO
+gboolean
+BodyPane :: on_verbose_tooltip_cb(GtkWidget  *widget,
+                                  gint        x,
+                                  gint        y,
+                                  gboolean    keyboard_tip,
+                                  GtkTooltip *tooltip,
+                                  gpointer    data)
+{
+  BodyPane* pane = static_cast<BodyPane*>(data);
+  if (!pane) return false;
+
+  gtk_tooltip_set_icon_from_stock (tooltip, GTK_STOCK_DIALOG_INFO, GTK_ICON_SIZE_DIALOG);
+
+  GPGDecErr& err = pane->_gpgerr;
+  GPGSignersInfo& info = err.signers;
+  if (err.no_sigs) return false;
+  if (info.signers.empty()) return false;
+
+  EvolutionDateMaker ed;
+
+  char buf[2048];
+  std::pair<std::string,std::string> name_and_email = get_email_address(info.signers[0].name);
+  g_snprintf(buf, sizeof(buf),
+             _("<u>This is a <b>PGP-Signed</b> message.</u>\n\n"
+             "<b>Signer:</b> %s ('%s')\n"
+             "<b>Valid until:</b> %s\n"
+             "<b>Created on:</b> %s"),
+             name_and_email.first.c_str(), name_and_email.second.c_str(),
+             (info.signers[0].never_expires ? _("always") : ed.get_date_string(info.signers[0].expires)),
+             ed.get_date_string(info.signers[0].created)
+             );
+
+  gtk_tooltip_set_markup (tooltip, buf);
+
+  return true;
+}
+#endif
+
 void
 BodyPane :: set_article (const Article& a)
 {
@@ -1180,7 +1283,13 @@ BodyPane :: set_article (const Article& a)
 
   if (_message)
     g_object_unref (_message);
+#ifdef HAVE_GMIME_CRYPTO
+  _gpgerr.clear();
+  _message = _cache.get_message (_article.get_part_mids(), _gpgerr);
+#else
   _message = _cache.get_message (_article.get_part_mids());
+#endif
+
   refresh ();
 
   _data.mark_read (_article);
@@ -1192,6 +1301,7 @@ BodyPane :: clear ()
   if (_message)
     g_object_unref (_message);
   _message = 0;
+
   refresh ();
 }
 
@@ -1239,12 +1349,22 @@ BodyPane :: expander_activated_idle (gpointer self_gpointer)
   self->_prefs.set_flag ("body-pane-headers-expanded", expanded);
   return false;
 }
+
 void
 BodyPane :: expander_activated_cb (GtkExpander*, gpointer self_gpointer)
 {
   g_idle_add (expander_activated_idle, self_gpointer);
 }
 
+void
+BodyPane :: verbose_clicked_cb (GtkWidget* w, GdkEvent  *event, gpointer self_gpointer)
+{
+  BodyPane *  self (static_cast<BodyPane*>(self_gpointer));
+  GtkExpander * ex (GTK_EXPANDER(self->_expander));
+  gtk_expander_set_expanded (ex, !gtk_expander_get_expanded (ex));
+  g_idle_add (expander_activated_idle, self_gpointer);
+}
+
 void
 BodyPane :: refresh_scroll_visible_state ()
 {
@@ -1255,7 +1375,7 @@ BodyPane :: refresh_scroll_visible_state ()
   _vscroll_visible = gtk_adjustment_get_page_size(adj) < gtk_adjustment_get_upper(adj);
 }
 
-// show*_cb exist to ensure that _hscroll_visible and _vscroll_visible
+// show*_cb exists to ensure that _hscroll_visible and _vscroll_visible
 // are initialized properly when the body pane becomes shown onscreen.
 
 gboolean
@@ -1286,6 +1406,7 @@ BodyPane :: text_size_allocated_idle_cb (gpointer pane)
 void
 BodyPane :: text_size_allocated_idle ()
 {
+
   // prevent oscillation
   const bool old_h (_hscroll_visible);
   const bool old_v (_vscroll_visible);
@@ -1308,6 +1429,7 @@ BodyPane :: text_size_allocated_idle ()
     if (gtk_text_iter_begins_tag (&iter, tag))
     {
       gtk_widget_get_allocation(_text, &aloc);
+      // BUG : resize
       resize_picture_at_iter (buf, &iter, fullsize, &aloc, tag);
     }
     if (!gtk_text_iter_forward_char (&iter))
@@ -1341,13 +1463,14 @@ BodyPane :: copy_url ()
   gtk_clipboard_set_text (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD),
                           _hover_url.c_str(), _hover_url.size());
 }
- 
+
 
 void
 BodyPane :: populate_popup_cb (GtkTextView *v, GtkMenu *m, gpointer pane)
 {
   static_cast<BodyPane*>(pane)->populate_popup(v, m);
 }
+
 void
 BodyPane :: populate_popup (GtkTextView *v G_GNUC_UNUSED, GtkMenu *m)
 {
@@ -1367,19 +1490,202 @@ BodyPane :: populate_popup (GtkTextView *v G_GNUC_UNUSED, GtkMenu *m)
   gtk_menu_shell_prepend (GTK_MENU_SHELL(m), mi);
 }
 
+namespace
+{
+  static gboolean
+  attachment_clicked_cb (GtkWidget *w, GdkEventButton *event, gpointer p)
+  {
+    BodyPane * bp = static_cast<BodyPane*>(p);
+    const gchar * fn = (char*)g_object_get_data (G_OBJECT(w), "filename");
+    if (!fn) return TRUE;
+
+    bp->_current_attachment = fn;
+
+    if (event->type == GDK_BUTTON_PRESS && event->button == 3)
+    {
+      gtk_menu_popup
+      (GTK_MENU(bp->_menu), NULL, NULL, NULL, NULL,
+      (event ? event->button : 0),
+      (event ? event->time : 0));
+    }
+
+    return TRUE;
+  }
+}
+
+void
+BodyPane :: menu_clicked (const MenuSelection& ms)
+{
+  std::vector<Article> copies;
+  copies.push_back(_article);
+  GroupPrefs _group_prefs;
+  SaveAttachmentsDialog * dialog = new SaveAttachmentsDialog (
+    _prefs, _group_prefs, _data, _data, _cache,
+    _data, _queue, get_window(_root),
+    _header_pane->get_group(), copies,
+    ms == MENU_SAVE_ALL ? TaskArticle::SAVE_ALL : TaskArticle::SAVE_AS,
+    _current_attachment);
+  gtk_widget_show (dialog->root());
+}
+
+void
+BodyPane :: menu_clicked_as_cb (GtkWidget* w, gpointer ptr)
+{
+  BodyPane* p = static_cast<BodyPane*>(ptr);
+  if (!p) return;
+  p->menu_clicked(MENU_SAVE_AS);
+}
+
+void
+BodyPane :: menu_clicked_all_cb (GtkWidget* w, gpointer ptr)
+{
+  BodyPane* p = static_cast<BodyPane*>(ptr);
+  if (!p) return;
+  p->menu_clicked(MENU_SAVE_ALL);
+}
+
+
+GtkWidget*
+BodyPane :: new_attachment (const char* filename)
+{
+  if (!filename) return 0;
+
+  GtkWidget* w = gtk_hbox_new(false, 0);
+  GtkWidget* attachment = gtk_label_new(filename);
+  GtkWidget * image = gtk_image_new_from_stock(GTK_STOCK_FILE, GTK_ICON_SIZE_MENU);
+
+  gtk_label_set_selectable (GTK_LABEL(attachment), true);
+
+  GtkWidget *event_box = gtk_event_box_new ();
+  gtk_container_add (GTK_CONTAINER (event_box), image);
+  g_object_set_data (G_OBJECT(event_box), "filename", (gpointer)filename);
+
+  gtk_box_pack_start (GTK_BOX(w), event_box, false, false, 0);
+  gtk_box_pack_start (GTK_BOX(w), attachment, false, false, 0);
+
+  g_signal_connect(event_box, "button_press_event", G_CALLBACK(attachment_clicked_cb), this);
+
+  return w;
+}
+
+
+void
+BodyPane :: clear_attachments()
+{
+  _cur_col = 0;
+  _cur_row = 0;
+  _attachments = 0;
+  _current_attachment = 0;
+
+  {
+    gtk_container_remove (GTK_CONTAINER (_att_frame), _att_toolbar);
+    if (G_IS_OBJECT(_att_toolbar)) g_object_unref(_att_toolbar);
+    (void)create_attachments_toolbar(_att_frame);
+  }
+
+}
+
+
+/// FIXME : shows only half the icon on gtk2+, gtk3+ works fine. hm....
+void
+BodyPane :: add_attachment_to_toolbar (const char* fn)
+{
+  if (!fn) return;
+
+  GtkWidget* w = new_attachment(fn);
+  gtk_widget_set_size_request(w, -1, 32);
+
+  ++_attachments;
+#if !GTK_CHECK_VERSION(3,0,0)
+
+  const guint cols(4);
+
+  if (_attachments % 4 == 0 && _attachments != 0)
+  {
+    ++_cur_row;
+    gtk_table_resize (GTK_TABLE(_att_toolbar), _cur_row, cols);
+    _cur_col = 0;
+  }
+
+  gtk_table_attach_defaults (GTK_TABLE(_att_toolbar), w, _cur_col, _cur_col+1, _cur_row,_cur_row+1);
+
+  ++_cur_col;
+
+#else
+
+  if (_attachments % 4 == 0 && _attachments != 0)
+  {
+    gtk_grid_insert_row (GTK_GRID(_att_toolbar), ++_cur_row);
+    _cur_col = 0;
+  }
+
+  gtk_grid_attach (GTK_GRID(_att_toolbar), w, _cur_col++, _cur_row, 1, 1);
+#endif  // 3.0.0
+
+  gtk_widget_show_all(_att_toolbar);
+}
+
+GtkWidget*
+BodyPane :: create_attachments_toolbar (GtkWidget* frame)
+{
+
+  _cur_col = 0;
+  _cur_row = 0;
+
+  GtkWidget * w;
+#if !GTK_CHECK_VERSION(3,0,0)
+  w = _att_toolbar = gtk_table_new(1,4,TRUE);
+  gtk_table_set_col_spacings (GTK_TABLE(w), PAD);
+  gtk_table_set_row_spacings (GTK_TABLE(w), PAD);
+#else
+  w = _att_toolbar = gtk_grid_new();
+  gtk_grid_insert_row (GTK_GRID(w), 0);
+  gtk_grid_set_column_spacing (GTK_GRID(w), 3);
+  gtk_grid_set_row_spacing (GTK_GRID (w), 4);
+#endif
+  gtk_container_add (GTK_CONTAINER (frame), w);
+  gtk_widget_show_all (frame);
+
+  return frame;
+}
 
 /***
 ****
 ***/
 
-BodyPane :: BodyPane (Data& data, ArticleCache& cache, Prefs& prefs):
+BodyPane :: BodyPane (Data& data, ArticleCache& cache, Prefs& prefs, GroupPrefs & gp, Queue& q, HeaderPane* hp):
   _prefs (prefs),
+  _group_prefs(gp),
+  _queue(q),
+  _header_pane(hp),
   _data (data),
   _cache (cache),
   _hscroll_visible (false),
   _vscroll_visible (false),
-  _message (0)
+  _message (0),
+#ifdef HAVE_GMIME_CRYPTO
+//  _gpgerr(GPG_DECODE),
+#endif
+  _attachments(0),
+  _current_attachment(0)
 {
+
+  GtkWidget * w, * l, * hbox;
+
+  for (guint i=0; i<NUM_ICONS; ++i)
+    icons[i].pixbuf = gdk_pixbuf_new_from_inline (-1, icons[i].pixbuf_txt, FALSE, 0);
+
+  // menu for popup menu for attachments
+  _menu = gtk_menu_new ();
+  l = gtk_menu_item_new_with_label(_("Save Attachment As..."));
+  g_signal_connect (l, "activate", G_CALLBACK(menu_clicked_as_cb), this);
+  gtk_menu_shell_append (GTK_MENU_SHELL(_menu), l);
+  l =  gtk_menu_item_new_with_label(_("Save All Attachments"));
+  _selection = MENU_SAVE_ALL;
+  g_signal_connect (l, "activate", G_CALLBACK(menu_clicked_all_cb), this);
+  gtk_menu_shell_append (GTK_MENU_SHELL(_menu),l);
+  gtk_widget_show_all(_menu);
+
   GtkWidget * vbox = gtk_vbox_new (false, PAD);
   gtk_container_set_resize_mode (GTK_CONTAINER(vbox), GTK_RESIZE_QUEUE);
 
@@ -1391,7 +1697,7 @@ BodyPane :: BodyPane (Data& data, ArticleCache& cache, Prefs& prefs):
   // tell the expander that it _wants_ to be very small, then it will still take
   // extra space given to it by its parent without asking for enough size to
   // fit the entire label.
-  GtkWidget * w = _expander = gtk_expander_new (NULL);
+  w = _expander = gtk_expander_new (NULL);
   gtk_widget_set_size_request (w, 50, -1);
   g_signal_connect (w, "activate", G_CALLBACK(expander_activated_cb), this);
   gtk_box_pack_start (GTK_BOX(vbox), w, false, false, 0);
@@ -1403,8 +1709,9 @@ BodyPane :: BodyPane (Data& data, ArticleCache& cache, Prefs& prefs):
   gtk_label_set_selectable (GTK_LABEL(_terse), true);
   gtk_label_set_ellipsize (GTK_LABEL(_terse), PANGO_ELLIPSIZE_MIDDLE);
   gtk_widget_show (_terse);
+  g_signal_connect (_terse, "button-press-event", G_CALLBACK(verbose_clicked_cb), this);
 
-  GtkWidget * hbox = _verbose = gtk_hbox_new (false, 0);
+  hbox = _verbose = gtk_hbox_new (false, 0);
   g_object_ref_sink (G_OBJECT(_verbose));
   w = _headers = gtk_label_new ("Headers");
   gtk_label_set_selectable (GTK_LABEL(_headers), TRUE);
@@ -1412,13 +1719,19 @@ BodyPane :: BodyPane (Data& data, ArticleCache& cache, Prefs& prefs):
   gtk_label_set_ellipsize (GTK_LABEL(w), PANGO_ELLIPSIZE_MIDDLE);
   gtk_label_set_use_markup (GTK_LABEL(w), true);
   gtk_box_pack_start (GTK_BOX(hbox), w, true, true, PAD_SMALL);
-  w = _xface = gtk_image_new ();
+
+  w = _xface = gtk_image_new();
   gtk_widget_set_size_request (w, 48, 48);
   gtk_box_pack_start (GTK_BOX(hbox), w, false, false, PAD_SMALL);
   w = _face = gtk_image_new ();
   gtk_widget_set_size_request (w, 48, 48);
   gtk_box_pack_start (GTK_BOX(hbox), w, false, false, PAD_SMALL);
   gtk_widget_show_all (_verbose);
+  g_signal_connect (_verbose, "button-press-event", G_CALLBACK(verbose_clicked_cb), this);
+#ifdef HAVE_GMIME_CRYPTO
+  gtk_widget_set_has_tooltip (_verbose, true);
+  g_signal_connect(_verbose,"query-tooltip",G_CALLBACK(on_verbose_tooltip_cb), this);
+#endif
 
   // setup
   _text = gtk_text_view_new ();
@@ -1436,6 +1749,10 @@ BodyPane :: BodyPane (Data& data, ArticleCache& cache, Prefs& prefs):
   gtk_widget_show_all (vbox);
   gtk_box_pack_start (GTK_BOX(vbox), _scroll, true, true, 0);
 
+  // add a toolbar for attachments
+  GtkWidget * frame = _att_frame = gtk_frame_new (_("Attachments"));
+  gtk_box_pack_start (GTK_BOX(vbox), create_attachments_toolbar(frame), false, false, 0);
+
   // set up the buffer tags
   _buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW(_text));
   set_text_buffer_tags (_buffer, _prefs);
@@ -1468,6 +1785,12 @@ BodyPane :: ~BodyPane ()
 
   if (_message)
     g_object_unref (_message);
+
+  for (int i=0; i<NUM_ICONS; ++i)
+    g_object_unref (icons[i].pixbuf);
+
+  foreach (std::set<char*>, _freeme, it)
+    g_free(*it);
 }
 
 
@@ -1549,7 +1872,7 @@ namespace
   }
 
   std::string get_header (GMimeMessage * msg,
-                          const char   * key, 
+                          const char   * key,
                           const char   * fallback_charset_1,
                           const char   * fallback_charset_2)
   {
@@ -1617,6 +1940,7 @@ BodyPane :: create_followup_or_reply (bool is_reply)
     const std::string newsgroups (get_header (_message, "Newsgroups",  message_charset, group_charset));
     const std::string fup_to     (get_header (_message, "Followup-To", message_charset, group_charset));
     const std::string reply_to   (get_header (_message, "Reply-To",    message_charset, group_charset));
+
     if (is_reply || fup_to=="poster") {
       const std::string& to (reply_to.empty() ? from : reply_to);
       pan_g_mime_message_add_recipients_from_string (msg, GMIME_RECIPIENT_TYPE_TO, to.c_str());
@@ -1634,7 +1958,6 @@ BodyPane :: create_followup_or_reply (bool is_reply)
     g_mime_message_set_subject (msg, val.c_str());
 
     // attribution lines
-
     const char * cpch = g_mime_object_get_header (_message_obj, "From");
     h = header_to_utf8 (cpch, message_charset, group_charset);
     g_mime_object_append_header (msg_obj, "X-Draft-Attribution-Author", h.c_str());
@@ -1705,7 +2028,9 @@ BodyPane :: create_followup_or_reply (bool is_reply)
     g_object_unref (wrapper);
     g_object_unref (part);
     g_object_unref (stream);
-//std::cerr << LINE_ID << " here is the modified clone\n [" << g_mime_object_to_string((GMimeObject *)msg) << ']' << std::endl;
+
+  //std::cerr << LINE_ID << " here is the modified clone\n [" << g_mime_object_to_string((GMimeObject *)msg) << ']' << std::endl;
+
   }
 
   return msg;
@@ -1740,6 +2065,7 @@ BodyPane :: on_prefs_flag_changed (const StringView& key, bool value G_GNUC_UNUS
     refresh_fonts ();
 
   if ((key=="wrap-article-body") || (key=="mute-quoted-text") ||
+      (key=="mute-signature") ||
       (key=="show-smilies-as-graphics") || (key=="show-all-headers") ||
       (key=="size-pictures-to-fit") || (key=="show-text-markup") ||
       (key=="highlight-urls") )
