@@ -20,7 +20,7 @@
 #ifndef _GroupPane_h_
 #define _GroupPane_h_
 
-#include <gtk/gtk.h>
+#include "gtk-compat.h"
 #include <pan/general/quark.h>
 #include <pan/data/data.h>
 #include <pan/gui/action-manager.h>
@@ -77,7 +77,7 @@ namespace pan
       void refresh_font ();
 
     public: // pretend these are private
-      void set_filter (const std::string& text);
+      void set_filter (const std::string& text, int mode);
       const ActionManager& _action_manager;
 
     private:
