@@ -20,7 +20,12 @@
 #include <config.h>
 extern "C" {
   #include <glib/gi18n.h>
-  #include <gtk/gtk.h>
+  #include <glib.h>
+  #include "gtk-compat.h"
+#ifdef HAVE_GTKSPELL
+  #include <gtkspell/gtkspell.h>
+  #include <enchant/enchant.h>
+#endif
 }
 #include <pan/general/debug.h>
 #include <pan/general/macros.h>
@@ -30,36 +35,98 @@ extern "C" {
 #include "hig.h"
 #include "pad.h"
 #include "pan-file-entry.h"
-#include "gtk_compat.h"
+#include "gtk-compat.h"
 
 using namespace pan;
 
+
 namespace
 {
+
+  struct Langs
+  {
+    GList* langs;
+  };
+
+  static void
+  dict_describe_cb(const char * const lang_tag,
+		 const char * const provider_name,
+		 const char * const provider_desc,
+		 const char * const provider_file,
+		 void * user_data)
+  {
+    Langs *langs = (Langs *)user_data;
+    langs->langs = g_list_insert_sorted(langs->langs, g_strdup(lang_tag), (GCompareFunc) strcmp);
+  }
+
+#ifdef HAVE_GTKSPELL
+  static EnchantBroker *broker = NULL;
+  static GList *langs = NULL;
+  static GtkTextView* view = NULL;
+  Langs l;
+#endif
+  void init_spell()
+  {
+#ifdef HAVE_GTKSPELL
+    view = GTK_TEXT_VIEW(gtk_text_view_new());
+    broker = enchant_broker_init();
+    l.langs = langs;
+    enchant_broker_list_dicts(broker, dict_describe_cb, &l);
+#endif
+  }
+
+  void deinit_spell()
+  {
+#ifdef HAVE_GTKSPELL
+    if (view) g_object_ref_sink(view);
+    if (broker) enchant_broker_free(broker);
+#endif
+  }
+
   void delete_dialog (gpointer castme)
   {
     delete static_cast<GroupPrefsDialog*>(castme);
   }
 }
-  
+
 void
 GroupPrefsDialog :: save_from_gui ()
 {
   // charset...
   const char * tmp = e_charset_combo_box_get_charset (E_CHARSET_COMBO_BOX(_charset));
-  _group_prefs.set_string (_group, "character-encoding", tmp);
+  foreach_const (quarks_v, _groups, it)
+    _group_prefs.set_string (*it, "character-encoding", tmp);
 
   // posting profile...
   std::string profile;
   if (gtk_widget_get_sensitive (_profile)) {
     char * pch (gtk_combo_box_text_get_active_text (GTK_COMBO_BOX_TEXT(_profile)));
-    _group_prefs.set_string (_group, "posting-profile", pch);
+    foreach_const (quarks_v, _groups, it)
+      _group_prefs.set_string (*it, "posting-profile", pch);
     g_free (pch);
   }
 
   // save path...
   const char * pch (file_entry_get (_save_path));
-  _group_prefs.set_string (_group, "default-group-save-path", pch);
+  foreach_const (quarks_v, _groups, it)
+    _group_prefs.set_string (*it, "default-group-save-path", pch);
+
+  _group_prefs.save () ;
+
+  // spellchecker language
+#ifdef HAVE_GTKSPELL
+  GtkTreeIter iter;
+  if (!gtk_combo_box_get_active_iter (GTK_COMBO_BOX(_spellchecker_language), &iter))
+		return;
+
+  gchar* name(0);
+	GtkTreeModel* model = gtk_combo_box_get_model (GTK_COMBO_BOX(_spellchecker_language));
+	gtk_tree_model_get (model, &iter, 0, &name, -1);
+
+  if (name) _group_prefs.set_string (_groups[0], "spellcheck-language", name);
+  g_free(name);
+#endif
+
 }
 
 void
@@ -75,7 +142,7 @@ namespace
 {
   GtkWidget*
   create_profiles_combo_box (const Data       & data,
-                             const Quark      & group,
+                             const quarks_v   & groups,
                              const GroupPrefs & group_prefs)
   {
     GtkWidget * w = gtk_combo_box_text_new ();
@@ -92,7 +159,7 @@ namespace
     }
     else
     {
-      const std::string s (group_prefs.get_string (group, "posting-profile", ""));
+      const std::string s (group_prefs.get_string (groups[0], "posting-profile", ""));
       int i(0), sel_index (0);
       foreach_const (unique_strings_t, profiles, it) {
         if (*it == s)
@@ -105,15 +172,62 @@ namespace
 
     return w;
   }
+
+  GtkWidget*
+  create_spellcheck_combo_box ( const Quark      & group,
+                                const GroupPrefs & group_prefs)
+  {
+    GtkWidget * w;
+
+#ifdef HAVE_GTKSPELL
+    init_spell();
+    deinit_spell();
+
+    GtkTreeModel *model;
+    GtkListStore * store = gtk_list_store_new (1, G_TYPE_STRING);
+    GtkTreeIter iter, storeit;
+    bool valid(false);
+
+
+    std::string lang = group_prefs.get_string(group, "spellcheck-language","");
+
+    while (l.langs)
+    {
+      gchar* data = (gchar*)l.langs->data;
+      if (data)
+      {
+        gtk_list_store_append (store, &iter);
+        gtk_list_store_set (store, &iter, 0, data, -1);
+        if (g_strcmp0 ((const char*)l.langs->data,lang.c_str())==0) { storeit = iter; valid=true; }
+      }
+      l.langs = l.langs->next;
+    }
+    model = GTK_TREE_MODEL(store);
+    w = gtk_combo_box_new_with_model (model);
+    g_object_unref(store);
+
+    GtkCellRenderer * renderer (gtk_cell_renderer_text_new ());
+    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (w), renderer, TRUE);
+    gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (w), renderer, "text", 0, NULL);
+
+    if (valid) gtk_combo_box_set_active_iter (GTK_COMBO_BOX(w), &storeit);
+
+    if (l.langs) g_list_free(l.langs);
+#endif
+    return w;
+  }
+
 }
 
-GroupPrefsDialog :: GroupPrefsDialog (Data         & data,
-                                      const Quark  & group,
-                                      GroupPrefs   & group_prefs,
-                                      GtkWindow    * parent_window):
-  _group (group),
+
+GroupPrefsDialog :: GroupPrefsDialog (Data            & data,
+                                      const quarks_v  & groups,
+                                      GroupPrefs      & group_prefs,
+                                      GtkWindow       * parent_window):
+  _groups (groups),
   _group_prefs (group_prefs)
 {
+
   GtkWidget * dialog = gtk_dialog_new_with_buttons (_("Pan: Group Preferences"),
                                                     parent_window,
                                                     GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -127,24 +241,44 @@ GroupPrefsDialog :: GroupPrefsDialog (Data         & data,
   int row (0);
   GtkWidget *t, *w, *l;
   t = HIG :: workarea_create ();
+
   char buf[512];
-  g_snprintf (buf, sizeof(buf), _("Properties for %s"), group.c_str());
+  if (groups.size() != 1)
+    g_snprintf (buf, sizeof(buf), _("Properties for Groups"));
+  else
+    g_snprintf (buf, sizeof(buf), _("Properties for %s"), groups[0].c_str());
+
   HIG::workarea_add_section_title (t, &row, buf);
-    HIG :: workarea_add_section_spacer (t, row, 3);
-    _charset = w = e_charset_combo_box_new( );
-    e_charset_combo_box_set_charset( E_CHARSET_COMBO_BOX(_charset), _group_prefs.get_string (group, "character-encoding", "UTF-8").c_str());
+    HIG :: workarea_add_section_spacer (t, row, 4);
+    w = _charset = e_charset_combo_box_new( );
+    const char* cs = _group_prefs.get_string (groups[0], "character-encoding", "UTF-8").c_str();
+    e_charset_combo_box_set_charset( E_CHARSET_COMBO_BOX(w), cs ? cs : "");
+
     HIG :: workarea_add_row (t, &row, _("Character _encoding:"), w);
+
     w = _save_path = file_entry_new (_("Directory for Saving Attachments"));
     char * pch = g_build_filename (g_get_home_dir(), "News", NULL);
-    const std::string dir (_group_prefs.get_string (_group, "default-group-save-path", pch));
+    std::string dir;
+    if (groups.size() != 1)
+      dir = pch;
+    else
+      dir = _group_prefs.get_string (groups[0], "default-group-save-path", pch);
     g_free (pch);
     file_entry_set (w, dir.c_str());
+
     HIG :: workarea_add_row (t, &row, _("Directory for _saving attachments:"), w);
-    w = _profile = create_profiles_combo_box (data, group, group_prefs);
+    w = _profile = create_profiles_combo_box (data, groups, group_prefs);
     l = HIG :: workarea_add_row (t, &row, _("Posting _profile:"), w);
+
     gtk_widget_set_sensitive (l, gtk_widget_get_sensitive(w));
+#ifdef HAVE_GTKSPELL
+    w = _spellchecker_language = create_spellcheck_combo_box ( groups[0], group_prefs);
+
+    HIG :: workarea_add_row (t, &row, _("Spellchecker _language:"), w);
+#endif
 
   gtk_widget_show_all (t);
   gtk_box_pack_start ( GTK_BOX( gtk_dialog_get_content_area( GTK_DIALOG( dialog))), t, true, true, 0);
   _root = dialog;
+
 }
