@@ -20,17 +20,20 @@
 #define _Pan_h_
 
 #include <pan/general/log.h>
+#include <pan/general/locking.h>
 #include <pan/general/progress.h>
 #include <pan/data/article-cache.h>
+#include <pan/data/encode-cache.h>
 #include <pan/tasks/queue.h>
-
+#include <pan/data/article.h>
+#include <pan/data/cert-store.h>
 #include <pan/gui/action-manager.h>
 #include <pan/gui/pan-ui.h>
 #include <pan/gui/prefs.h>
 #include <pan/gui/group-prefs.h>
 #include <pan/gui/wait.h>
 
-#include "gtk_compat.h"
+#include "gtk-compat.h"
 
 namespace pan
 {
@@ -50,12 +53,27 @@ namespace pan
     private Log::Listener,
     private Progress::Listener,
     private Queue::Listener,
-    private Prefs::Listener
+    private Prefs::Listener,
+    private CertStore::Listener,
+    private Data::Listener
   {
+
     public:
-      GUI (Data& data, Queue&, ArticleCache&, Prefs&, GroupPrefs&);
+      GUI (Data& data, Queue&, Prefs&, GroupPrefs&);
       virtual ~GUI ();
       GtkWidget* root () { return _root; }
+      typedef std::vector<std::string> strings_t;
+
+      struct VerifyData
+      {
+#ifdef HAVE_GNUTLS
+        gnutls_x509_crt_t cert;
+#endif
+        std::string server;
+        std::string cert_name;
+        int nr;
+        GUI* gui;
+      };
 
     public: // ActionManager
       virtual bool is_action_active (const char * action_name) const;
@@ -112,18 +130,28 @@ namespace pan
       virtual void do_read_previous_thread ();
       virtual void do_read_parent_article ();
       virtual void do_show_servers_dialog ();
+      virtual void do_show_sec_dialog ();
       virtual void do_show_selected_article_info ();
       virtual void do_plonk ();
       virtual void do_watch ();
       virtual void do_ignore ();
+      virtual void do_flag ();
+      virtual void do_flag_off ();
+      virtual void do_next_flag ();
+      virtual void do_last_flag ();
+      virtual void do_mark_all_flagged ();
+      virtual void do_invert_selection ();
       virtual void do_show_score_dialog ();
       virtual void do_show_new_score_dialog ();
       virtual void do_cancel_article ();
       virtual void do_supersede_article ();
       virtual void do_delete_article ();
+      virtual bool deletion_confirmation_dialog();
       virtual void do_clear_article_cache ();
       virtual void do_mark_article_read ();
       virtual void do_mark_article_unread ();
+      virtual void do_mark_thread_read ();
+      virtual void do_mark_thread_unread ();
       virtual void do_post ();
       virtual void do_followup_to ();
       virtual void do_reply_to ();
@@ -142,6 +170,7 @@ namespace pan
       virtual void do_match_only_binary_articles (bool);
       virtual void do_match_only_my_articles (bool);
       virtual void do_match_only_unread_articles (bool);
+      virtual void do_enable_toggle_rules (bool enable);
       virtual void do_match_on_score_state (int);
       virtual void do_show_matches (const Data::ShowType);
       virtual void do_read_selected_group ();
@@ -153,10 +182,16 @@ namespace pan
       virtual void do_refresh_groups ();
       virtual void do_subscribe_selected_groups ();
       virtual void do_unsubscribe_selected_groups ();
+#ifdef HAVE_GNUTLS
+      void do_show_cert_failed_dialog(VerifyData* data);
+      bool confirm_accept_new_cert_dialog(GtkWindow*, gnutls_x509_crt_t, const Quark&);
+#endif
+      void step_bookmarks(int step);
+      void do_read_or_save_articles ();
 
     public:
       static std::string prompt_user_for_save_path (GtkWindow * parent, const Prefs& prefs);
-	  static std::string prompt_user_for_filename  (GtkWindow * parent, const Prefs& prefs);
+      static std::string prompt_user_for_filename  (GtkWindow * parent, const Prefs& prefs);
 
     private: // Queue::Listener
       friend class Queue;
@@ -168,8 +203,11 @@ namespace pan
       virtual void on_queue_size_changed (Queue&, int active, int total);
       virtual void on_queue_online_changed (Queue&, bool online);
       virtual void on_queue_error (Queue&, const StringView& message);
-
-
+#ifdef HAVE_GNUTLS
+    private:  // CertStore::Listener
+      virtual void on_verify_cert_failed(gnutls_x509_crt_t, std::string, int);
+      virtual void on_valid_cert_added (gnutls_x509_crt_t, std::string);
+#endif
     private: // Log::Listener
       virtual void on_log_entry_added (const Log::Entry& e);
       virtual void on_log_cleared () {}
@@ -188,8 +226,10 @@ namespace pan
       Data& _data;
       Queue& _queue;
       ArticleCache& _cache;
+      EncodeCache& _encode_cache;
       Prefs& _prefs;
       GroupPrefs& _group_prefs;
+      CertStore& _certstore;
 
     private:
       GtkWidget * _root;
@@ -212,7 +252,6 @@ namespace pan
       GtkWidget * _taskbar;
       std::vector<ProgressView*> _views;
       std::list<Task*> _active_tasks;
-
       std::string _charset;
 
       void set_charset (const StringView& v);
@@ -234,6 +273,16 @@ namespace pan
       static void add_widget (GtkUIManager*, GtkWidget*, gpointer);
       static void server_list_dialog_destroyed_cb (GtkWidget*, gpointer);
       void server_list_dialog_destroyed (GtkWidget*);
+      static void sec_dialog_destroyed_cb (GtkWidget*, gpointer);
+      void sec_dialog_destroyed (GtkWidget*);
+      static void prefs_dialog_destroyed_cb (GtkWidget * w, gpointer self);
+      void prefs_dialog_destroyed (GtkWidget* w);
+      int score_int_from_string(std::string val, const char* rules[]);
+#ifdef HAVE_GNUTLS
+      static gboolean show_cert_failed_cb(gpointer gp);
+#endif
+    public:
+      GtkUIManager* get_ui_manager() { return _ui_manager; }
   };
 }
 
