@@ -20,24 +20,28 @@
 extern "C" {
   #include <config.h>
   #include <glib/gi18n.h>
-  #include <gtk/gtk.h>
+  #include "gtk-compat.h"
+  #include "gtk-compat.h"
 }
 #include <cctype>
 #include <cmath>
 #include <algorithm>
 #include <pan/general/debug.h>
 #include <pan/general/e-util.h>
+#include <pan/general/utf8-utils.h>
 #include <pan/general/log.h>
 #include <pan/general/macros.h>
 #include <pan/general/quark.h>
 #include <pan/usenet-utils/filter-info.h>
+#include <pan/usenet-utils/rules-info.h>
+#include <pan/usenet-utils/mime-utils.h>
 #include <pan/data/article.h>
 #include <pan/data/data.h>
 #include <pan/icons/pan-pixbufs.h>
 #include "header-pane.h"
 #include "render-bytes.h"
 #include "tango-colors.h"
-#include "gtk_compat.h"
+#include "gtk-compat.h"
 
 using namespace pan;
 
@@ -75,14 +79,19 @@ namespace
   {
     ICON_READ,
     ICON_UNREAD,
+
     ICON_COMPLETE,
     ICON_COMPLETE_READ,
+
     ICON_INCOMPLETE,
     ICON_INCOMPLETE_READ,
+
     ICON_CACHED,
     ICON_QUEUED,
     ICON_ERROR,
     ICON_EMPTY,
+    ICON_FLAGGED,
+    ICON_GET_FLAGGED,
     ICON_QTY
    };
 
@@ -92,14 +101,19 @@ namespace
   } _icons[ICON_QTY] = {
     { icon_article_read,           0 },
     { icon_article_unread,         0 },
+
     { icon_binary_complete,        0 },
     { icon_binary_complete_read,   0 },
+
     { icon_binary_incomplete,      0 },
     { icon_binary_incomplete_read, 0 },
+
     { icon_disk,                   0 },
     { icon_bluecheck,              0 },
     { icon_x,                      0 },
-    { icon_empty,                  0 }
+    { icon_empty,                  0 },
+    { icon_red_flag,               0 },
+    { icon_get_flagged,            0 }
   };
 
   int
@@ -126,17 +140,22 @@ namespace
   }
 
   int
-  get_article_action (const ArticleCache  & cache,
+  get_article_action (const Article       * article,
+                      const ArticleCache  & cache,
                       const Queue         & queue,
                       const Quark         & message_id)
   {
     int offset (ICON_EMPTY);
-      
+
     if (queue.contains (message_id))
       offset = ICON_QUEUED;
     else if (cache.contains (message_id))
       offset = ICON_CACHED;
 
+    if (article)
+      if (article->flag)
+        offset = ICON_FLAGGED;
+
     return offset;
   }
 };
@@ -229,6 +248,24 @@ HeaderPane :: render_score (GtkTreeViewColumn * ,
                           NULL);
 }
 
+void
+HeaderPane :: render_author (GtkTreeViewColumn * ,
+                            GtkCellRenderer   * renderer,
+                            GtkTreeModel      * model,
+                            GtkTreeIter       * iter,
+                            gpointer            user_data)
+{
+
+  const HeaderPane * self (static_cast<HeaderPane*>(user_data));
+  const Row * row (dynamic_cast<Row*>(self->_tree_store->get_row (iter)));
+
+  const Article * a (self->get_article (model, iter));
+
+  char* ret = __g_mime_iconv_strdup(conv, a->author.c_str());
+  if (ret) g_object_set (renderer, "text", ret, NULL);
+  g_free(ret);
+}
+
 void
 HeaderPane :: render_bytes (GtkTreeViewColumn * ,
                             GtkCellRenderer   * renderer,
@@ -241,6 +278,7 @@ HeaderPane :: render_bytes (GtkTreeViewColumn * ,
   g_object_set (renderer, "text", pan::render_bytes(bytes), NULL);
 }
 
+
 struct HeaderPane::CountUnread: public PanTreeStore::WalkFunctor
 {
   const Row * top_row;
@@ -258,6 +296,7 @@ struct HeaderPane::CountUnread: public PanTreeStore::WalkFunctor
   }
 };
 
+
 void
 HeaderPane :: render_subject (GtkTreeViewColumn * ,
                               GtkCellRenderer   * renderer,
@@ -266,6 +305,7 @@ HeaderPane :: render_subject (GtkTreeViewColumn * ,
                               gpointer            user_data)
 {
   const HeaderPane * self (static_cast<HeaderPane*>(user_data));
+  const Prefs & p(self->_prefs);
   const Row * row (dynamic_cast<Row*>(self->_tree_store->get_row (iter)));
 
   CountUnread counter (row);
@@ -274,10 +314,15 @@ HeaderPane :: render_subject (GtkTreeViewColumn * ,
   const bool bold (!row->is_read);
 
   const Article * a (self->get_article (model, iter));
-  const char * text (a->subject.c_str());
+
+  char* ret = __g_mime_iconv_strdup(conv, a->subject.c_str());
+  std::string res;
+  if (ret) res = ret;
+  g_free(ret);
+
   char buf[512];
 
-  bool underlined (false);
+  bool unread (false);
 
   if (counter.unread_children)
   {
@@ -288,17 +333,22 @@ HeaderPane :: render_subject (GtkTreeViewColumn * ,
     gtk_tree_path_free (path);
 
     if (!expanded) {
-      underlined = row->is_read;
-      snprintf (buf, sizeof(buf), "%s (%lu)", text, counter.unread_children);
-      text = buf;
+      unread = row->is_read;
+      snprintf (buf, sizeof(buf), "%s (%lu)", res.c_str(), counter.unread_children);
+      res = buf;
     }
   }
 
+  std::string def_bg, def_fg;
+  def_fg = p.get_color_str_wo_fallback("color-read-fg");
+  def_bg = p.get_color_str_wo_fallback("color-read-bg");
+
   g_object_set (renderer,
-    "text", text,
+    "text", res.c_str(),
     "weight", (bold ? PANGO_WEIGHT_BOLD : PANGO_WEIGHT_NORMAL),
-    "underline", (underlined ? PANGO_UNDERLINE_SINGLE : PANGO_UNDERLINE_NONE),
-    NULL);
+    "foreground", unread ? (def_fg.empty() ? NULL : def_fg.c_str()) : NULL,
+    "background", unread ? (def_bg.empty() ? NULL : def_bg.c_str()) : NULL, NULL);
+
 }
 
 HeaderPane::Row*
@@ -312,14 +362,14 @@ HeaderPane::Row*
 HeaderPane :: create_row (const EvolutionDateMaker & e,
                           const Article            * a)
 {
-  const int action (get_article_action (_cache, _queue, a->message_id));
+  const int action (get_article_action (a, _cache, _queue, a->message_id));
   const int state (get_article_state (_data, a));
   char * date_str (e.get_date_string (a->time_posted));
   Row * row = new Row (_data, a, date_str, action, state);
 
   std::pair<mid_to_row_t::iterator,bool> result (_mid_to_row.insert (row));
   g_assert (result.second);
-  
+
   return row;
 }
 
@@ -362,8 +412,6 @@ HeaderPane :: column_compare_func (GtkTreeModel  * model,
 {
   int ret (0);
   const PanTreeStore * store (reinterpret_cast<PanTreeStore*>(model));
-  //const Row& row_a (*dynamic_cast<const Row*>(store->get_row (iter_a)));
-  //const Row& row_b (*dynamic_cast<const Row*>(store->get_row (iter_b)));
   const Row& row_a (*static_cast<const Row*>(store->get_row (iter_a)));
   const Row& row_b (*static_cast<const Row*>(store->get_row (iter_b)));
 
@@ -467,7 +515,7 @@ HeaderPane :: build_model (const Quark               & group,
 
 namespace
 {
-  void save_sort_order (Prefs& prefs, PanTreeStore* store)
+  void save_sort_order (const Quark& group, GroupPrefs& prefs, PanTreeStore* store)
   {
     g_assert (store);
     g_assert (GTK_IS_TREE_SORTABLE(store));
@@ -475,8 +523,8 @@ namespace
     gint sort_column (0);
     GtkSortType sort_type;
     gtk_tree_sortable_get_sort_column_id (GTK_TREE_SORTABLE(store), &sort_column,  &sort_type);
-    prefs.set_int ("header-pane-sort-column", sort_column);
-    prefs.set_flag ("header-pane-sort-ascending", sort_type==GTK_SORT_ASCENDING);
+    prefs.set_int  (group, "header-pane-sort-column", sort_column);
+    prefs.set_flag (group, "header-pane-sort-ascending", sort_type==GTK_SORT_ASCENDING);
   }
 }
 
@@ -485,16 +533,16 @@ HeaderPane :: rebuild ()
 {
   quarks_t selectme;
   if (1) {
-    const articles_t old_selection (get_full_selection ());
-    foreach_const (articles_t, old_selection, it)
+    const articles_set old_selection (get_full_selection ());
+    foreach_const (articles_set, old_selection, it)
       selectme.insert ((*it)->message_id);
   }
 
   _mid_to_row.clear ();
   _tree_store = build_model (_group, _atree, NULL);
 
-  const bool sort_ascending = _prefs.get_flag ("header-pane-sort-ascending", false);
-  int sort_column = _prefs.get_int ("header-pane-sort-column", COL_DATE);
+  const bool sort_ascending = _group_prefs.get_flag (_group, "header-pane-sort-ascending", false);
+  int sort_column = _group_prefs.get_int (_group, "header-pane-sort-column", COL_DATE);
   if (sort_column<0 || sort_column>=N_COLUMNS) // safeguard against odd settings
     sort_column = COL_DATE;
   gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE(_tree_store),
@@ -529,7 +577,7 @@ HeaderPane :: set_group (const Quark& new_group)
 
     if (_tree_store)
     {
-      save_sort_order (_prefs, _tree_store);
+      save_sort_order (get_group(), _group_prefs, _tree_store);
       _mid_to_row.clear ();
       _tree_store = 0;
       gtk_tree_view_set_model (GTK_TREE_VIEW(_tree_view), NULL);
@@ -540,9 +588,13 @@ HeaderPane :: set_group (const Quark& new_group)
     delete _atree;
     _atree = 0;
 
+    char * pch = g_build_filename (g_get_home_dir(), "News", NULL);
+    Quark path(_group_prefs.get_string (_group, "default-group-save-path", pch));
+    g_free(pch);
+
     if (!_group.empty())
     {
-      _atree = _data.group_get_articles (new_group, _show_type, &_filter);
+      _atree = _data.group_get_articles (new_group, path, _show_type, &_filter,&_rules);
       _atree->add_listener (this);
 
       rebuild ();
@@ -622,7 +674,8 @@ HeaderPane :: select_message_id (const Quark& mid, bool do_scroll)
   HeaderPane::Row * row = get_row (mid);
   GtkTreePath * path (_tree_store->get_path(row));
   GtkTreeView * view (GTK_TREE_VIEW(_tree_view));
-  gtk_tree_view_expand_to_path (view, path);
+  const bool expand (_prefs.get_flag("expand-selected-articles",false));
+  if (expand) gtk_tree_view_expand_to_path (view, path);
   GtkTreeSelection * sel (gtk_tree_view_get_selection (view));
   gtk_tree_selection_select_path (sel, path);
   if (do_scroll) {
@@ -682,9 +735,14 @@ HeaderPane :: on_tree_change (const Data::ArticleTree::Diffs& diffs)
       Row * child (get_row (it->first));
       tmp[parent].push_back (child);
     }
+
+    g_object_ref(G_OBJECT (_tree_store));
+    gtk_tree_view_set_model(GTK_TREE_VIEW(_tree_view), NULL);
     _tree_store->insert_sorted (tmp);
+    gtk_tree_view_set_model(GTK_TREE_VIEW(_tree_view), GTK_TREE_MODEL (_tree_store));
+    g_object_unref(G_OBJECT(_tree_store));
   }
-     
+
   // reparent...
   if (do_thread && !diffs.reparented.empty()) {
     PanTreeStore::parent_to_children_t tmp;
@@ -708,7 +766,12 @@ HeaderPane :: on_tree_change (const Data::ArticleTree::Diffs& diffs)
                          keep.begin(), keep.end(),
                          inserter (kill, kill.begin()), o);
     g_assert (keep.size() + kill.size() == _mid_to_row.size());
+
+    g_object_ref(G_OBJECT (_tree_store));
+    gtk_tree_view_set_model(GTK_TREE_VIEW(_tree_view), NULL);
     _tree_store->remove (kill);
+    gtk_tree_view_set_model(GTK_TREE_VIEW(_tree_view), GTK_TREE_MODEL (_tree_store));
+    g_object_unref(G_OBJECT(_tree_store));
     _mid_to_row.get_container().swap (keep);
   }
 
@@ -744,6 +807,23 @@ HeaderPane :: get_first_selected_article () const
    return a;
 }
 
+Article*
+HeaderPane :: get_first_selected_article ()
+{
+   Article * a (0);
+   std::set<const Article*> articles (get_full_selection ());
+   if (!articles.empty())
+     a = (Article*)*articles.begin ();
+   return a;
+}
+
+const guint
+HeaderPane :: get_full_selection_rows_num() const
+{
+  return (gtk_tree_selection_count_selected_rows(gtk_tree_view_get_selection(GTK_TREE_VIEW(_tree_view))));
+}
+
+
 void
 HeaderPane :: get_full_selection_v_foreach (GtkTreeModel * model,
                                             GtkTreePath  * ,
@@ -776,10 +856,73 @@ HeaderPane :: get_full_selection () const
   return articles;
 }
 
+void
+HeaderPane :: mark_all_flagged ()
+{
+
+  GtkTreeIter iter;
+  GtkTreeModel * model(gtk_tree_view_get_model(GTK_TREE_VIEW(_tree_view)));
+  GtkTreeSelection * sel (gtk_tree_view_get_selection (GTK_TREE_VIEW(_tree_view)));
+  gtk_tree_selection_unselect_all(sel);
+  gtk_tree_model_get_iter_first (model, &iter);
+  walk_and_collect_flagged(model, &iter, sel);
+
+}
+
+void
+HeaderPane :: invert_selection ()
+{
+
+  GtkTreeIter iter;
+  GtkTreeModel * model(gtk_tree_view_get_model(GTK_TREE_VIEW(_tree_view)));
+  GtkTreeSelection * sel (gtk_tree_view_get_selection (GTK_TREE_VIEW(_tree_view)));
+  gtk_tree_model_get_iter_first (model, &iter);
+  walk_and_invert_selection (model, &iter, sel);
+
+}
+
+
+void
+HeaderPane :: walk_and_invert_selection (GtkTreeModel          * model,
+                                         GtkTreeIter           * cur,
+                                         GtkTreeSelection      * ref) const
+{
+   for (;;) {
+    const bool selected(gtk_tree_selection_iter_is_selected (ref, cur));
+    if (selected)
+      gtk_tree_selection_unselect_iter(ref,cur);
+    else
+      gtk_tree_selection_select_iter(ref,cur);
+    GtkTreeIter child;
+    if (gtk_tree_model_iter_children (model, &child, cur))
+      walk_and_invert_selection (model, &child, ref);
+    if (!gtk_tree_model_iter_next (model, cur))
+      break;
+  }
+
+}
+
+void
+HeaderPane :: walk_and_collect_flagged (GtkTreeModel  * model,
+                                GtkTreeIter           * cur,
+                                GtkTreeSelection      * setme) const
+{
+   for (;;) {
+    const Article * a(get_article (model, cur));
+    if (a->get_flag()) gtk_tree_selection_select_iter(setme,cur);
+    GtkTreeIter child;
+    if (gtk_tree_model_iter_children (model, &child, cur))
+      walk_and_collect_flagged (model, &child, setme);
+    if (!gtk_tree_model_iter_next (model, cur))
+      break;
+  }
+
+}
+
 void
 HeaderPane :: walk_and_collect (GtkTreeModel          * model,
                                 GtkTreeIter           * cur,
-                                articles_t            & setme) const
+                                articles_set          & setme) const
 {
   for (;;) {
     setme.insert (get_article (model, cur));
@@ -795,6 +938,7 @@ namespace {
   struct NestedData {
     const HeaderPane * pane;
     articles_t articles;
+    bool mark_all; /* for mark_article_(un)read and mark_thread_(un)read */
   };
 }
 
@@ -804,13 +948,15 @@ HeaderPane :: get_nested_foreach (GtkTreeModel  * model,
                                   GtkTreeIter   * iter,
                                   gpointer        data) const
 {
-  articles_t& articles (*static_cast<articles_t*>(data));
+  NestedData& ndata (*static_cast<NestedData*>(data));
+  articles_set& articles (ndata.articles);
   articles.insert (get_article (model, iter));
   const bool expanded (gtk_tree_view_row_expanded (GTK_TREE_VIEW(_tree_view), path));
   GtkTreeIter child;
-  if (!expanded && gtk_tree_model_iter_children(model,&child,iter))
+  if ((!expanded || ndata.mark_all) && gtk_tree_model_iter_children(model,&child,iter))
     walk_and_collect (model, &child, articles);
 }
+
 void
 HeaderPane :: get_nested_foreach_static (GtkTreeModel* model,
                                          GtkTreePath* path,
@@ -818,14 +964,15 @@ HeaderPane :: get_nested_foreach_static (GtkTreeModel* model,
                                          gpointer data)
 {
   NestedData& ndata (*static_cast<NestedData*>(data));
-  ndata.pane->get_nested_foreach (model, path, iter, &ndata.articles);
+  ndata.pane->get_nested_foreach (model, path, iter, &ndata);
 }
 
-articles_t
-HeaderPane :: get_nested_selection () const
+articles_set
+HeaderPane :: get_nested_selection (bool do_mark_all) const
 {
   NestedData data;
   data.pane = this;
+  data.mark_all = do_mark_all;
   GtkTreeSelection * selection (gtk_tree_view_get_selection (GTK_TREE_VIEW(_tree_view)));
   gtk_tree_selection_selected_foreach (selection, get_nested_foreach_static, &data);
   return data.articles;
@@ -904,6 +1051,33 @@ namespace
   }
 }
 
+namespace
+{
+  static gboolean return_pressed_download_all (gpointer data)
+  {
+    HeaderPane * self (static_cast<HeaderPane*>(data));
+    // TODO move this into headerpane
+    self->_gui.do_read_or_save_articles();
+    return false;
+
+  }
+}
+
+gboolean
+HeaderPane :: on_keyboard_button_pressed ( GtkWidget *widget, GdkEventKey *event, gpointer data)
+{
+  if (event->type == GDK_KEY_PRESS)
+  {
+    if (event->keyval == GDK_KEY_Return)
+    {
+      g_idle_add(return_pressed_download_all, data);
+      return TRUE;
+    }
+  }
+
+  return FALSE;
+}
+
 gboolean
 HeaderPane :: on_button_pressed (GtkWidget * treeview, GdkEventButton *event, gpointer userdata)
 {
@@ -916,7 +1090,7 @@ HeaderPane :: on_button_pressed (GtkWidget * treeview, GdkEventButton *event, gp
     GtkTreeSelection * selection = gtk_tree_view_get_selection(tv);
     GtkTreePath *path;
     if (gtk_tree_view_get_path_at_pos (tv,
-                                       (gint) event->x, 
+                                       (gint) event->x,
                                        (gint) event->y,
                                        &path, NULL, NULL, NULL))
     {
@@ -936,7 +1110,7 @@ HeaderPane :: on_button_pressed (GtkWidget * treeview, GdkEventButton *event, gp
            && (event->send_event == false)
            && (event->window == gtk_tree_view_get_bin_window (tv))
            && !(event->state & (GDK_SHIFT_MASK|GDK_CONTROL_MASK|GDK_MOD1_MASK))
-           && (self->get_full_selection_v().size() == 1u))
+           && (self->get_full_selection_rows_num() == 1u))
   {
     GtkTreePath * path;
     GtkTreeViewColumn * col;
@@ -1024,11 +1198,62 @@ namespace
     AUTHOR,
     MESSAGE_ID
   };
+
+}
+
+#define RANGE 4998
+std::pair<int,int>
+HeaderPane :: get_int_from_rules_str(std::string val)
+{
+  std::pair<int,int> res;
+  if (val == "new")     { res.first = 0;      res.second = 0; }
+  if (val == "never")   { res.first = 10;     res.second = 5; } // inversed, so never true
+  if (val == "watched") { res.first = 9999;   res.second = 99999; }
+  if (val == "high")    { res.first = 5000;   res.second = 5000+RANGE; }
+  if (val == "medium")  { res.first = 1;      res.second = 1+RANGE; }
+  if (val == "low")     { res.first = -9998;  res.second = -1; }
+  if (val == "ignored") { res.first = -9999;  res.second = -99999; }
+  return res;
+}
+
+void
+HeaderPane :: rebuild_rules (bool enable)
+{
+
+  if (!enable)
+  {
+    _rules.clear();
+    return;
+  }
+
+  RulesInfo &r (_rules);
+  r.set_type_aggregate_and ();
+  RulesInfo tmp;
+
+  std::pair<int,int> res;
+
+  res = get_int_from_rules_str(_prefs.get_string("rules-delete-value", "never"));
+  tmp.set_type_delete_b (res.first, res.second);
+  r._aggregates.push_back (tmp);
+
+  res = get_int_from_rules_str(_prefs.get_string("rules-mark-read-value", "never"));
+  tmp.set_type_mark_read_b (res.first, res.second);
+  r._aggregates.push_back (tmp);
+
+  res = get_int_from_rules_str(_prefs.get_string("rules-autocache-value", "never"));
+  tmp.set_type_autocache_b (res.first, res.second);
+  r._aggregates.push_back (tmp);
+
+  res = get_int_from_rules_str(_prefs.get_string("rules-auto-dl-value", "never"));
+  tmp.set_type_dl_b (res.first, res.second);
+   r._aggregates.push_back (tmp);
+
 }
 
 void
 HeaderPane :: rebuild_filter (const std::string& text, int mode)
 {
+
   TextMatch::Description d;
   d.negate = false;
   d.case_sensitive = false;
@@ -1036,6 +1261,7 @@ HeaderPane :: rebuild_filter (const std::string& text, int mode)
   d.text = text;
 
   FilterInfo &f (_filter);
+
   f.set_type_aggregate_and ();
 
   // entry field filter...
@@ -1174,6 +1400,24 @@ HeaderPane :: filter (const std::string& text, int mode)
   }
 }
 
+void
+HeaderPane :: rules(bool enable)
+{
+  rebuild_rules(enable);
+
+  if (_atree)
+  {
+    _wait.watch_cursor_on ();
+
+    if (_rules._aggregates.empty())
+      _atree->set_rules();
+    else
+      _atree->set_rules(_show_type, &_rules);
+
+    _wait.watch_cursor_off ();
+  }
+}
+
 namespace
 {
   // the text typed by the user.
@@ -1402,11 +1646,12 @@ HeaderPane :: build_tree_columns ()
         "ypad", 0,
         NULL));
       ellipsize_if_supported (G_OBJECT(r));
-      col = gtk_tree_view_column_new_with_attributes (_("Author"), r, "text", COL_SHORT_AUTHOR, NULL);
+      col = gtk_tree_view_column_new_with_attributes (_("Author"), r, NULL);
       gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_FIXED);
       gtk_tree_view_column_set_fixed_width (col, _prefs.get_int (width_key, 133));
       gtk_tree_view_column_set_resizable (col, true);
       gtk_tree_view_column_set_sort_column_id (col, COL_SHORT_AUTHOR);
+      gtk_tree_view_column_set_cell_data_func (col, r, render_author, this, 0);
       gtk_tree_view_append_column (tree_view, col);
     }
     else if (name == "lines")
@@ -1485,7 +1730,7 @@ HeaderPane :: ~HeaderPane ()
   _cache.remove_listener (this);
   _queue.remove_listener (this);
   _prefs.remove_listener (this);
-  _data.remove_listener (this);
+  _data.remove_listener  (this);
 
   // save the column widths
   GList * columns = gtk_tree_view_get_columns (GTK_TREE_VIEW(_tree_view));
@@ -1501,6 +1746,7 @@ HeaderPane :: ~HeaderPane ()
 
   for (guint i=0; i<ICON_QTY; ++i)
     g_object_unref (G_OBJECT(_icons[i].pixbuf));
+
 }
 
 GtkWidget*
@@ -1569,6 +1815,8 @@ HeaderPane :: on_selection_changed_idle (gpointer self_gpointer)
     "show-selected-article-info",
     "mark-article-read",
     "mark-article-unread",
+    "mark-thread-read",
+    "mark-thread-unread",
     "watch-thread",
     "ignore-thread",
     "plonk",
@@ -1594,18 +1842,22 @@ HeaderPane :: HeaderPane (ActionManager       & action_manager,
                           Queue               & queue,
                           ArticleCache        & cache,
                           Prefs               & prefs,
-                          WaitUI              & wait):
+                          GroupPrefs          & group_prefs,
+                          WaitUI              & wait,
+                          GUI                 & gui):
   _action_manager (action_manager),
   _data (data),
   _queue (queue),
   _prefs (prefs),
+  _group_prefs (group_prefs),
   _wait (wait),
   _atree (0),
   _root (0),
   _tree_view (0),
   _tree_store (0),
   _selection_changed_idle_tag (0),
-  _cache (cache)
+  _cache (cache),
+  _gui (gui)
 {
   // init the icons
   for (guint i=0; i<ICON_QTY; ++i)
@@ -1634,6 +1886,10 @@ HeaderPane :: HeaderPane (ActionManager       & action_manager,
 
   g_signal_connect (w, "button-release-event", G_CALLBACK(on_button_pressed), this);
   g_signal_connect (w, "button-press-event", G_CALLBACK(on_button_pressed), this);
+
+  /* intercept ENTER for selection of multiple articles */
+  g_signal_connect (w, "key-press-event", G_CALLBACK(on_keyboard_button_pressed), this);
+
   g_signal_connect (w, "row-collapsed", G_CALLBACK(row_collapsed_cb), NULL);
   g_signal_connect (w, "row-expanded", G_CALLBACK(row_expanded_cb), NULL);
   g_signal_connect (w, "popup-menu", G_CALLBACK(on_popup_menu), this);
@@ -1649,6 +1905,7 @@ HeaderPane :: HeaderPane (ActionManager       & action_manager,
   _root = scroll;
 
   search_activate (this); // calls rebuild_filter
+  rules(_prefs._rules_enabled);
 
   _data.add_listener (this);
   _prefs.add_listener (this);
@@ -1789,6 +2046,14 @@ namespace
     virtual bool operator()(const Article&) const { return true; }
   };
 
+  struct ArticleIsFlagged: public ArticleTester {
+    virtual ~ArticleIsFlagged () {}
+    const Article* article;
+    ArticleIsFlagged (const Article* a) : article(a) {}
+    virtual bool operator()(const Article& a) const
+      { return a.get_flag() && a.message_id != article->message_id; }
+  };
+
   struct ArticleIsParentOf: public ArticleTester {
     virtual ~ArticleIsParentOf () {}
     ArticleIsParentOf (const Data::ArticleTree& tree, const Article* a) {
@@ -1869,7 +2134,7 @@ namespace
 }
 
 /**
-*** 
+***
 **/
 
 void
@@ -1942,6 +2207,7 @@ HeaderPane :: action_next_if (const ArticleTester& test, RowActionFunctor& actio
 void
 HeaderPane :: read_next_if (const ArticleTester& test)
 {
+
   GtkTreeView * v (GTK_TREE_VIEW(_tree_view));
   ReadFunctor read (v, _action_manager);
   action_next_if (test, read);
@@ -1950,12 +2216,32 @@ HeaderPane :: read_next_if (const ArticleTester& test)
 void
 HeaderPane :: read_prev_if (const ArticleTester & test)
 {
+
   GtkTreeView * v (GTK_TREE_VIEW(_tree_view));
   GtkTreeModel * m (GTK_TREE_MODEL(_tree_store));
   ReadFunctor read (v, _action_manager);
   next_iterator (v, m, TreeIteratorPrev(), test, read);
 }
 
+void
+HeaderPane :: select_next_if (const ArticleTester& test)
+{
+
+  GtkTreeView * v (GTK_TREE_VIEW(_tree_view));
+  SelectFunctor sel (v);
+  action_next_if (test, sel);
+}
+
+void
+HeaderPane :: select_prev_if (const ArticleTester& test)
+{
+
+  GtkTreeView * v (GTK_TREE_VIEW(_tree_view));
+  GtkTreeModel * m (GTK_TREE_MODEL(_tree_store));
+  SelectFunctor sel (v);
+  next_iterator (v, m, TreeIteratorPrev(), test, sel);
+}
+
 void
 HeaderPane :: read_next_unread_article ()
 {
@@ -2006,6 +2292,23 @@ HeaderPane :: read_parent_article ()
 ****
 ***/
 
+void
+HeaderPane :: move_to_next_bookmark(int step)
+{
+  const bool forward(step == 1);
+  Article * a (get_first_selected_article());
+  if (!a) return;
+
+  if (forward)
+    select_next_if (ArticleIsFlagged(get_first_selected_article()));
+  else
+    select_prev_if (ArticleIsFlagged(get_first_selected_article()));
+}
+
+/***
+****
+***/
+
 void
 HeaderPane :: refresh_font ()
 {
@@ -2046,11 +2349,19 @@ HeaderPane :: rebuild_article_action (const Quark& message_id)
 {
   Row * row (get_row (message_id));
   if (row) {
-    row->action = get_article_action (_cache, _queue, message_id);
+    row->action = get_article_action (row->article, _cache, _queue, message_id);
     _tree_store->row_changed (row);
   }
 }
 
+void
+HeaderPane :: on_article_flag_changed (articles_t& a, const Quark& group)
+{
+  g_return_if_fail(!a.empty());
+  foreach_const (articles_t, a, it)
+    rebuild_article_action ((*it)->message_id);
+}
+
 void
 HeaderPane :: on_queue_tasks_added (Queue& queue, int index, int count)
 {
@@ -2068,6 +2379,7 @@ HeaderPane :: on_queue_task_removed (Queue&, Task& task, int)
   if (ta)
     rebuild_article_action (ta->get_article().message_id);
 }
+
 void
 HeaderPane :: on_cache_added (const Quark& message_id)
 {
@@ -2075,6 +2387,7 @@ HeaderPane :: on_cache_added (const Quark& message_id)
   q.insert(message_id);
   _data.rescore_articles ( _group, q );
   rebuild_article_action (message_id);
+
 }
 void
 HeaderPane :: on_cache_removed (const quarks_t& message_ids)
@@ -2126,10 +2439,10 @@ struct HeaderPane::SimilarWalk: public PanTreeStore::WalkFunctor
       // strip out frequent substrings that tend to skew string_likeness too high
       static const char * const frequent_substrings [] = { "mp3", "gif", "jpg", "jpeg", "yEnc" };
       for (size_t i=0; i!=G_N_ELEMENTS(frequent_substrings); ++i) {
-	std::string::size_type pos;
-	const char * needle (frequent_substrings[i]);
-	while (((pos = sa.find (needle))) != std::string::npos) sa.erase (pos, strlen(needle));
-	while (((pos = sb.find (needle))) != std::string::npos) sb.erase (pos, strlen(needle));
+        std::string::size_type pos;
+        const char * needle (frequent_substrings[i]);
+        while (((pos = sa.find (needle))) != std::string::npos) sa.erase (pos, strlen(needle));
+        while (((pos = sb.find (needle))) != std::string::npos) sb.erase (pos, strlen(needle));
       }
 
       // strip out non-alpha characters
@@ -2144,11 +2457,11 @@ struct HeaderPane::SimilarWalk: public PanTreeStore::WalkFunctor
       const bool is_long_string (min_len >= 30);
       double min_closeness;
       if (is_short_string)
-	min_closeness = 0.6;
+        min_closeness = 0.6;
       else if (is_long_string)
-	min_closeness = 0.5;
+        min_closeness = 0.5;
       else
-	min_closeness = 0.55;
+        min_closeness = 0.55;
 
       return string_likeness (sa, sb) >= min_closeness;
     }
@@ -2160,30 +2473,29 @@ struct HeaderPane::SimilarWalk: public PanTreeStore::WalkFunctor
 
       if (!a.strchr(' ')) // only one word, so count common characters
       {
-	int common_chars = 0;
+        int common_chars = 0;
 
-	foreach_const (StringView, a, it) {
-	  const char * pos = b.strchr (*it);
-	  if (pos) {
-	    ++common_chars;
-	    b.eat_chars (pos - b.str);
-	  }
-	}
+        foreach_const (StringView, a, it) {
+          const char * pos = b.strchr (*it);
+          if (pos) {
+            ++common_chars;
+            b.eat_chars (pos - b.str);
+          }
+        }
 
-	retval = (double)common_chars / a.len;
+        retval = (double)common_chars / a.len;
       }
-      else // more than one word, so count common words
+        else // more than one word, so count common words
       {
-	StringView tok;
-	int str1_words(0), common_words(0);
-	while (a.pop_token (tok)) {
-	  ++str1_words;
-	  const char *pch = b.strstr (tok);
-	  if (pch) ++common_words;
-	}
-	retval = (double)common_words / str1_words;
+        StringView tok;
+        int str1_words(0), common_words(0);
+        while (a.pop_token (tok)) {
+          ++str1_words;
+          const char *pch = b.strstr (tok);
+          if (pch) ++common_words;
+        }
+        retval = (double)common_words / str1_words;
       }
-
       return retval;
     }
 };
