@@ -20,18 +20,21 @@
 #ifndef _HeaderPane_h_
 #define _HeaderPane_h_
 
-#include <gtk/gtk.h>
+#include "gtk-compat.h"
 #include <pan/general/e-util.h>
 #include <pan/general/macros.h> // for UNUSED
 #include <pan/data/article-cache.h>
 #include <pan/data/data.h>
 #include <pan/usenet-utils/filter-info.h>
+#include <pan/usenet-utils/rules-info.h>
 #include <pan/usenet-utils/gnksa.h>
 #include <pan/tasks/queue.h>
 #include <pan/gui/action-manager.h>
 #include <pan/gui/pan-tree.h>
 #include <pan/gui/prefs.h>
+#include <pan/gui/group-prefs.h>
 #include <pan/gui/wait.h>
+#include <pan/gui/gui.h>
 
 namespace pan
 {
@@ -62,6 +65,9 @@ namespace pan
     virtual void operator() (GtkTreeModel* model, GtkTreeIter* iter, const Article& a) = 0;
   };
 
+  typedef std::vector<const Article*> articles_t;
+  typedef std::set<const Article*> articles_set;
+
   /**
    * Header Pane in the main window of Pan's GUI.
    * @ingroup GUI
@@ -74,7 +80,7 @@ namespace pan
     private ArticleCache::Listener
   {
     public:
-      HeaderPane (ActionManager&, Data& data, Queue&, ArticleCache&, Prefs&, WaitUI&);
+      HeaderPane (ActionManager&, Data& data, Queue&, ArticleCache&, Prefs&, GroupPrefs&, WaitUI&, GUI&);
       ~HeaderPane ();
 
     public:
@@ -99,19 +105,27 @@ namespace pan
       void read_previous_thread ();
       void read_parent_article ();
 
+      void mark_all_flagged ();
+      void invert_selection ();
+      void move_to_next_bookmark(int);
+
     private:
       void action_next_if (const ArticleTester& test, RowActionFunctor& action);
       void read_next_if (const ArticleTester&);
       void read_prev_if (const ArticleTester&);
+      void select_next_if (const ArticleTester& test);
+      void select_prev_if (const ArticleTester& test);
 
     public:
       GtkWidget* root () { return _root; }
       GtkWidget* get_default_focus_widget() { return _tree_view; }
       GtkWidget* create_filter_entry ();
       const Article* get_first_selected_article () const;
+      Article* get_first_selected_article ();
       std::set<const Article*> get_full_selection () const;
       std::vector<const Article*> get_full_selection_v () const;
-      std::set<const Article*> get_nested_selection () const;
+      const guint get_full_selection_rows_num () const;
+      std::set<const Article*> get_nested_selection (bool do_mark_all) const;
       bool set_group (const Quark& group);
       const Quark& get_group () { return _group; }
 
@@ -136,6 +150,9 @@ namespace pan
       virtual void on_prefs_string_changed (const StringView&, const StringView&);
       virtual void on_prefs_color_changed  (const StringView&, const GdkColor&) {}
 
+    public:
+      virtual void on_article_flag_changed (articles_t& a, const Quark& group);
+
     private:
       virtual void on_queue_task_active_changed (Queue&, Task&, bool active UNUSED) { }
       virtual void on_queue_tasks_added (Queue&, int index, int count);
@@ -150,6 +167,7 @@ namespace pan
       virtual void on_cache_added (const Quark& mid);
       virtual void on_cache_removed (const quarks_t& mids);
 
+
     public: // pretend it's private
       ActionManager& _action_manager;
 
@@ -290,6 +308,7 @@ namespace pan
       Data& _data;
       Queue& _queue;
       Prefs& _prefs;
+      GroupPrefs& _group_prefs;
       WaitUI& _wait;
       Quark _group;
       Data::ArticleTree * _atree;
@@ -297,19 +316,25 @@ namespace pan
       GtkWidget * _tree_view;
       PanTreeStore * _tree_store;
       FilterInfo _filter;
+      RulesInfo _rules;
       Data::ShowType _show_type;
       guint _selection_changed_idle_tag;
 
     private:
       void rebuild_filter (const std::string&, int);
+      void rebuild_rules (bool enable=false);
+      std::pair <int,int> get_int_from_rules_str(std::string val);
       void refresh_font ();
 
     public: // public so that anonymous namespace can reach -- don't call
       void filter (const std::string& text, int mode);
+      void rules (bool enable=false);
       static void do_popup_menu (GtkWidget*, GdkEventButton*, gpointer);
       static void on_row_activated (GtkTreeView*, GtkTreePath*, GtkTreeViewColumn*, gpointer);
       static gboolean on_button_pressed (GtkWidget*, GdkEventButton*, gpointer);
+      static gboolean on_keyboard_button_pressed ( GtkWidget *widget, GdkEventKey *event, gpointer data);
       ArticleCache& _cache;
+      GUI& _gui;
 
     private:
       void get_nested_foreach (GtkTreeModel*, GtkTreePath*, GtkTreeIter*, gpointer) const;
@@ -322,7 +347,10 @@ namespace pan
       class CountUnread;
       class RowInserter;
       class SimilarWalk;
-      void walk_and_collect (GtkTreeModel*, GtkTreeIter*, std::set<const Article*>&) const;
+      void walk_and_collect (GtkTreeModel*, GtkTreeIter*, articles_set&) const;
+      void walk_and_collect_flagged  (GtkTreeModel*, GtkTreeIter*, GtkTreeSelection*) const;
+      void walk_and_invert_selection (GtkTreeModel*, GtkTreeIter*, GtkTreeSelection*) const;
+
     private:
       typedef void RenderFunc (GtkTreeViewColumn*, GtkCellRenderer*, GtkTreeModel*, GtkTreeIter*, gpointer);
       static RenderFunc render_action;
@@ -330,6 +358,7 @@ namespace pan
       static RenderFunc render_score;
       static RenderFunc render_bytes;
       static RenderFunc render_subject;
+      static RenderFunc render_author;
 
     private:
       Row* get_row (const Quark& message_id);
