@@ -22,10 +22,11 @@
 
 #include <map>
 #include <set>
+#include <iostream>
 #include <string>
 #include <vector>
 #include <pan/general/string-view.h>
-#include <gtk/gtk.h>
+#include "gtk-compat.h"
 
 namespace pan
 {
@@ -42,6 +43,7 @@ namespace pan
         virtual void on_prefs_int_changed (const StringView& key, int color) = 0;
         virtual void on_prefs_string_changed (const StringView& key, const StringView& value) = 0;
         virtual void on_prefs_color_changed (const StringView& key, const GdkColor& color) = 0;
+        virtual void on_prefs_hotkey_changed (const StringView& key, const StringView& value) {}
       };
       void add_listener (Listener* l) { _listeners.insert(l); }
       void remove_listener (Listener* l) {_listeners.erase(l); }
@@ -67,6 +69,10 @@ namespace pan
         for (listeners_t::iterator it(_listeners.begin()), end(_listeners.end()); it!=end; )
           (*it++)->on_prefs_color_changed (key, value);
       }
+      void fire_hotkey_changed (const StringView& key, const StringView& value) {
+        for (listeners_t::iterator it(_listeners.begin()), end(_listeners.end()); it!=end; )
+          (*it++)->on_prefs_hotkey_changed (key, value);
+      }
 
     public:
       static void start_element (GMarkupParseContext *context,
@@ -80,6 +86,7 @@ namespace pan
       bool get_flag (const StringView& key, bool fallback) const;
       void set_flag (const StringView& key, bool);
       int get_int (const StringView& key, int fallback) const;
+      int get_int_min  (const StringView& key, int fallback) const;
       void set_int (const StringView& key, int);
 
       std::string get_string (const StringView& key, const StringView& fallback) const;
@@ -89,9 +96,13 @@ namespace pan
       void set_color (const StringView& key, const StringView& color_str);
       std::string get_color_str (const StringView& key, const GdkColor& fallback) const;
       std::string get_color_str (const StringView& key, const StringView& fallback_str) const;
+      std::string get_color_str_wo_fallback (const StringView& key) const;
       GdkColor get_color (const StringView& key, const GdkColor& fallback) const;
       GdkColor get_color (const StringView& key, const StringView& fallback_str) const;
 
+//      void set_hotkey (const StringView& key, const StringView& value);
+//      std::string get_hotkey (const StringView& key) const;
+
       void set_window (const StringView& key, GtkWindow* window,
                        int default_x, int default_y,
                        int default_width, int default_height);
@@ -101,7 +112,7 @@ namespace pan
       bool get_geometry (const StringView&, int&, int&, int&, int&) const;
 
     public:
-      Prefs () {}
+      Prefs () { _rules_changed = false; }
       virtual void save () const {}
       virtual ~Prefs () {}
 
@@ -128,6 +139,11 @@ namespace pan
       mutable colors_t _colors;
       typedef std::map<std::string,int> ints_t;
       mutable ints_t _ints;
+
+
+    public:
+      bool _rules_changed;
+      bool _rules_enabled;
   };
 }
 
