@@ -20,17 +20,35 @@
 #ifndef SERVER_UI_H
 #define SERVER_UI_H
 
-#include <gtk/gtkwindow.h>
+#include <set>
+#include "gtk-compat.h"
 #include <pan/data/data.h>
 #include <pan/tasks/queue.h>
 
 namespace pan
 {
+
+  typedef std::set<std::string> strings_t;
+
   /** @ingroup GUI */
   GtkWidget* server_edit_dialog_new (Data&, Queue&, GtkWindow*, const Quark& server);
 
   /** @ingroup GUI */
   GtkWidget* server_list_dialog_new (Data&, Queue&, GtkWindow*);
+
+  /** @ingroup GUI */
+  GtkWidget* sec_dialog_new (Data& data, Queue& queue, GtkWindow* parent);
+
+  /** @ingroup GUI */
+  std::string
+  import_sec_from_disk_dialog_new (Data& data, Queue& queue, GtkWindow * window);
+
+  /** @ingroup GUI */
+  void render_cert_flag (GtkTreeViewColumn * ,
+                         GtkCellRenderer   * ,
+                         GtkTreeModel      * ,
+                         GtkTreeIter       * ,
+                         gpointer            );
 }
 
 #endif
