@@ -19,7 +19,7 @@
 
 #include <config.h>
 #include <pan/general/debug.h>
-#include <pan/general/foreach.h>
+#include <pan/general/macros.h>
 #include <pan/general/messages.h>
 #include <pan/data/server-info.h>
 #include "queue.h"
@@ -29,23 +29,34 @@
 ****
 ***/
 
+// TODO Mulithreading downloads!
+
 using namespace pan;
 
 Queue :: Queue (ServerInfo         & server_info,
                 TaskArchive        & archive,
-                Socket::Creator    * socket_creator,
+                SocketCreator      * socket_creator,
+                CertStore          & certstore,
                 WorkerPool         & pool,
-                bool                 online):
+                bool                 online,
+                int                  save_delay_secs):
   _server_info (server_info),
   _is_online (online),
   _socket_creator (socket_creator),
   _worker_pool (pool),
   _decoder (pool),
+  _encoder (pool),
   _decoder_task (0),
+  _encoder_task (0),
+  _save_delay_secs (save_delay_secs),
   _needs_saving (false),
   _last_time_saved (0),
-  _archive (archive)
+  _archive (archive),
+  _certstore(certstore),
+  _uploads_total(0),
+  _downloads_total(0)
 {
+
   tasks_t tasks;
   _archive.load_tasks (tasks);
   add_tasks (tasks, BOTTOM);
@@ -61,6 +72,8 @@ Queue :: ~Queue ()
     delete it->second;
   _pools.clear ();
 
+  clean_n_save();
+
   foreach (TaskSet, _tasks, it)
     delete *it;
 }
@@ -74,7 +87,7 @@ Queue :: on_pool_has_nntp_available (const Quark& server)
 }
 
 void
-Queue :: on_pool_error (const Quark& server, const StringView& message)
+Queue :: on_pool_error (const Quark& server UNUSED, const StringView& message)
 {
   fire_queue_error (message);
 }
@@ -91,7 +104,7 @@ Queue :: get_pool (const Quark& servername)
   }
   else // have to build one
   {
-    pool = new NNTP_Pool (servername, _server_info, _socket_creator);
+    pool = new NNTP_Pool (servername, _server_info, _socket_creator, _certstore);
     pool->add_listener (this);
     _pools[servername] = pool;
   }
@@ -102,19 +115,35 @@ Queue :: get_pool (const Quark& servername)
 /***
 ****
 ***/
+void
+Queue :: clean_n_save ()
+{
+
+  const tasks_t tmp (_tasks.begin(), _tasks.end());
+  // remove completed tasks.
+  foreach_const (tasks_t, tmp, it) {
+    Task * task  (*it);
+    const Task::State& state (task->get_state());
+    if (state._work==Task::COMPLETED || _removing.count(task))
+      remove_task (task);
+  }
+
+  // maybe save the task list.
+  // only do it once in awhile to prevent thrashing.
+  const time_t now (time(0));
+  if (_needs_saving && _last_time_saved<(now-_save_delay_secs)) {
+    _archive.save_tasks (tmp);
+    _needs_saving = false;
+    _last_time_saved = time(0);
+  }
+}
 
 void
 Queue :: upkeep ()
 {
-  // maybe save the task list.
-  // only do it once in awhile to prevent thrashing.
+  clean_n_save();
+
   const tasks_t tmp (_tasks.begin(), _tasks.end());
-  const time_t now (time(0));
-  if (_needs_saving && _last_time_saved<(now-10)) {
-    _archive.save_tasks (tmp);
-    _needs_saving = false;
-    _last_time_saved = now;
-  }
 
   // do upkeep on the first queued task.
   // the CPU goes crazy if we run upkeep on _all_ queued tasks,
@@ -123,20 +152,12 @@ Queue :: upkeep ()
   // ref #352170, #354779
   foreach_const (tasks_t, tmp, it) {
     Task * task (*it);
-    if (task->get_state()._work == Task::NEED_NNTP) {
+    if (task->get_state()._work == Task::NEED_NNTP ) {
       process_task (task);
       break;
     }
   }
 
-  // remove completed tasks.
-  foreach_const (tasks_t, tmp, it) {
-    Task * task  (*it);
-    const Task::State& state (task->get_state());
-    if (state._work==Task::COMPLETED || _removing.count(task))
-      remove_task (task);
-  }
-
   // upkeep on running tasks... this lets us pop open
   // extra connections if the task can handle >1 connection
   std::set<Task*> active;
@@ -145,6 +166,10 @@ Queue :: upkeep ()
   foreach (std::set<Task*>, active, it)
     process_task (*it);
 
+  //upkeep on paused upload slots
+//  foreach (std::set<TaskUpload*>, _uploads, it)
+//    process_task (*it);
+
   // idle socket upkeep
   foreach (pools_t, _pools, it)
     it->second->idle_upkeep ();
@@ -189,6 +214,8 @@ Queue :: get_task_counts (int& active, int& total)
     active_tasks.insert (it->second);
   if (_decoder_task)
     active_tasks.insert (_decoder_task);
+  if (_encoder_task)
+    active_tasks.insert (_encoder_task);
   active = active_tasks.size ();
   total = _tasks.size ();
 }
@@ -203,6 +230,16 @@ Queue :: give_task_a_decoder (Task * task)
   task->give_decoder (this, &_decoder); // it's active now...
 }
 
+void
+Queue :: give_task_a_encoder (Task * task)
+{
+  const bool was_active (task_is_active (task));
+  _encoder_task = task;
+  if (!was_active)
+    fire_task_active_changed (task, true);
+  task->give_encoder (this, &_encoder); // it's active now...
+}
+
 void
 Queue :: give_task_a_connection (Task * task, NNTP * nntp)
 {
@@ -214,10 +251,36 @@ Queue :: give_task_a_connection (Task * task, NNTP * nntp)
   task->give_nntp (this, nntp);
 }
 
+void
+Queue :: give_task_an_upload_slot (TaskUpload* task)
+{
+  int max (_server_info.get_server_limits(task->_server));
+  if (_uploads.size() < max)
+  {
+    _uploads.insert(task);
+    task->wakeup();
+    fire_task_active_changed (task, true);
+    process_task(task);
+  }
+}
+
+void
+Queue :: give_task_a_download_slot (TaskArticle* task)
+{
+  int max (8);//DBG!!(_server_info.get_server_limits(task->_server));
+  if (_downloads.size() < max)
+  {
+    _downloads.insert(task);
+    task->wakeup();
+    fire_task_active_changed (task, true);
+    process_task(task);
+  }
+}
+
 void
 Queue :: process_task (Task * task)
 {
-  pan_return_if_fail (task!=0);
+  pan_return_if_fail (task != 0);
 
   debug ("in process_task with a task of type " << task->get_type());
 
@@ -246,30 +309,59 @@ Queue :: process_task (Task * task)
   else if (state._work == Task::WORKING)
   {
     debug ("working");
-    // do nothing
+  }
+  else if (state._work == Task::PAUSED)
+  {
+    debug("paused");
+    TaskUpload* t = dynamic_cast<TaskUpload*>(task);
+    if (t)
+      give_task_an_upload_slot(t);
+
+//    TaskArticle* t2 = dynamic_cast<TaskArticle*>(task);
+//    if (t2)
+//      give_task_a_download_slot(t2);
+
   }
   else if (state._work == Task::NEED_DECODER)
   {
+
     if (!_decoder_task)
       give_task_a_decoder (task);
   }
+  else if (state._work == Task::NEED_ENCODER)
+  {
+    if (!_encoder_task)
+      give_task_a_encoder (task);
+  }
+
   else while (_is_online && (state._work == Task::NEED_NNTP))
   {
+    debug("online");
     // make the requests...
     const Task::State::unique_servers_t& servers (state._servers);
     foreach_const (Task::State::unique_servers_t, servers, it)
+    {
+      if (_certstore.in_blacklist(*it)) continue;
       get_pool(*it).request_nntp (_worker_pool);
+    }
 
     Quark server;
     if (!find_best_server (servers, server))
+    {
+      debug("break");
       break;
+    }
 
     NNTP * nntp (get_pool(server).check_out ());
     if (!nntp)
+    {
+      debug("break");
       break;
+    }
 
     give_task_a_connection (task, nntp);
   }
+  debug("end loop");
 }
 
 /***
@@ -290,6 +382,20 @@ Queue :: find_first_task_needing_decoder ()
   return 0;
 }
 
+Task*
+Queue :: find_first_task_needing_encoder ()
+{
+  foreach (TaskSet, _tasks, it) {
+    const Task::State& state ((*it)->get_state ());
+    if  ((state._work == Task::NEED_ENCODER)
+      && (!_stopped.count (*it))
+      && (!_removing.count (*it)))
+      return *it;
+  }
+
+  return 0;
+}
+
 Task*
 Queue :: find_first_task_needing_server (const Quark& server)
 {
@@ -342,21 +448,21 @@ Queue :: find_best_server (const Task::State::unique_servers_t& servers, Quark&
 ***/
 
 void
-Queue :: on_set_items_added (TaskSet& container, TaskSet::items_t& tasks, int pos)
+Queue :: on_set_items_added (TaskSet& container UNUSED, TaskSet::items_t& tasks, int pos)
 {
   _needs_saving = true;
   fire_tasks_added (pos, tasks.size());
 }
 
 void
-Queue :: on_set_item_removed (TaskSet& container, Task*& task, int pos)
+Queue :: on_set_item_removed (TaskSet& container UNUSED, Task*& task, int pos)
 {
   _needs_saving = true;
   fire_task_removed (task, pos);
 }
 
 void
-Queue :: on_set_item_moved (TaskSet& container, Task*& task, int new_pos, int old_pos)
+Queue :: on_set_item_moved (TaskSet& container UNUSED, Task*& task, int new_pos, int old_pos)
 {
   _needs_saving = true;
   fire_task_moved (task, new_pos, old_pos);
@@ -505,6 +611,9 @@ Queue :: task_is_active (const Task * task) const
   if (task && task==_decoder_task)
     return true;
 
+  if (task && task==_encoder_task)
+    return true;
+
   bool task_has_nntp (false);
   foreach_const (nntp_to_task_t, _nntp_to_task, it)
     if ((task_has_nntp = task==it->second))
@@ -536,10 +645,27 @@ Queue :: remove_task (Task * task)
     TaskArticle * ta (dynamic_cast<TaskArticle*>(task));
     if (ta)
       _mids.erase (ta->get_article().message_id);
-
     _stopped.erase (task);
     _removing.erase (task);
     _tasks.remove (index);
+
+    // manually upkeep on ONE new taskupload to keep the queue going
+    TaskUpload * t  (dynamic_cast<TaskUpload*>(task));
+    if (t)
+    {
+      int max (_server_info.get_server_limits(t->_server));
+      _uploads.erase(t);
+      const tasks_t tmp (_tasks.begin(), _tasks.end());
+      foreach_const (tasks_t, tmp, it) {
+        Task * task (*it);
+        if (task->get_state()._work == Task::PAUSED)
+        {
+          give_task_an_upload_slot(dynamic_cast<TaskUpload*>(*it));
+          break;
+        }
+      }
+    }
+
     delete task;
   }
 
@@ -629,7 +755,7 @@ Queue :: check_in (NNTP * nntp, Health nntp_health)
 }
 
 void
-Queue :: check_in (Decoder* decoder, Task* task)
+Queue :: check_in (Decoder* decoder UNUSED, Task* task)
 {
   // take care of our decoder counting...
   _decoder_task = 0;
@@ -652,6 +778,30 @@ Queue :: check_in (Decoder* decoder, Task* task)
   process_task (task);
 }
 
+void
+Queue :: check_in (Encoder* encoder UNUSED, Task* task)
+{
+  // take care of our decoder counting...
+  _encoder_task = 0;
+
+  // notify the listeners if the task isn't active anymore...
+  if (!task_is_active (task))
+    fire_task_active_changed (task, false);
+
+  // if the task hit an error, fire an error message
+  const Task::State state (task->get_state ());
+  if (state._health == ERR_LOCAL)
+    fire_queue_error ("");
+
+  // pass our worker thread on to another task
+  Task * next = find_first_task_needing_encoder ();
+  if (next && (next!=task))
+    process_task (next);
+
+  // what to do now with this task...
+  process_task (task);
+}
+
 void
 Queue :: move_up (const tasks_t& tasks)
 {
@@ -775,11 +925,11 @@ Queue :: get_stats (unsigned long   & queued_count,
     Task * task (*it);
 
     const Queue::TaskState state (tasks.get_state (task));
-    if (state == Queue::RUNNING || state == Queue::DECODING)
+    if (state == Queue::RUNNING || state == Queue::DECODING || state == Queue::ENCODING)
       ++running_count;
     else if (state == Queue::STOPPED)
       ++stopped_count;
-    else if (state == Queue::QUEUED || state == Queue::QUEUED_FOR_DECODE)
+    else if (state == Queue::QUEUED || state == Queue::QUEUED_FOR_DECODE || state == Queue::QUEUED_FOR_ENCODE)
       ++queued_count;
 
     if (state==Queue::RUNNING || state==Queue::QUEUED)
