@@ -24,11 +24,13 @@
 #include <config.h>
 #include <algorithm>
 #include <cassert>
-extern "C" {
-#include <glib/gi18n.h>
+extern "C"
+{
+  #include <glib/gi18n.h>
 }
 #include <pan/general/debug.h>
 #include <pan/general/file-util.h>
+#include <pan/general/utf8-utils.h>
 #include <pan/general/log.h>
 #include <pan/general/macros.h>
 #include <pan/usenet-utils/mime-utils.h>
@@ -49,13 +51,35 @@ namespace
     std::string stripped;
     mime::remove_multipart_from_subject (article.subject.c_str(), stripped);
 
+    const char* str = stripped.c_str();
+    iconv_t c = iconv_open("UTF-8","UTF-8");
+    char * res = __g_mime_iconv_strdup(c, str);
+    iconv_close(c);
+
     char buf[1024];
     if (save)
-      snprintf (buf, sizeof(buf), _("Saving %s"), stripped.c_str());
+      snprintf (buf, sizeof(buf), _("Saving %s"), res);
     else
-      snprintf (buf, sizeof(buf), _("Reading %s"), stripped.c_str());
+      snprintf (buf, sizeof(buf), _("Reading %s"), res);
+
+    g_free(res);
+
     return std::string (buf);
   }
+
+  std::string get_groups_str(const Article& a)
+  {
+    std::string r;
+    quarks_t groups;
+    int cnt(1);
+    foreach_const (Xref, a.xref, xit)
+    {
+      r += xit->group.to_string();
+      if (cnt != a.xref.size() && a.xref.size() != 1) r+=", ";
+      ++cnt;
+    }
+    return r;
+  }
 }
 
 TaskArticle :: TaskArticle (const ServerRank          & server_rank,
@@ -65,7 +89,9 @@ TaskArticle :: TaskArticle (const ServerRank          & server_rank,
                             ArticleRead               & read,
                             Progress::Listener        * listener,
                             SaveMode                    save_mode,
-                            const Quark               & save_path):
+                            const Quark               & save_path,
+                            const char                * filename,
+                            const SaveOptions         & options):
   Task (save_path.empty() ? "BODIES" : "SAVE", get_description (article, !save_path.empty())),
   _save_path (save_path),
   _server_rank (server_rank),
@@ -75,8 +101,12 @@ TaskArticle :: TaskArticle (const ServerRank          & server_rank,
   _time_posted (article.time_posted),
   _save_mode (save_mode),
   _decoder(0),
-  _decoder_has_run (false)
+  _decoder_has_run (false),
+  _groups(get_groups_str(article)),
+  _attachment(filename),
+  _options(options)
 {
+
   cache.reserve (article.get_part_mids());
 
   if (listener != 0)
@@ -119,10 +149,15 @@ TaskArticle :: TaskArticle (const ServerRank          & server_rank,
   // initialize our progress status...
   init_steps (all_bytes);
   set_step (all_bytes - need_bytes);
+  const char *artsub(article.subject.c_str());
   if (save_path.empty())
-    set_status (article.subject.c_str());
+    set_status (artsub);
   else
-    set_status_va (_("Saving %s"), article.subject.c_str());
+    set_status_va (_("Saving %s"), artsub);
+
+  char buf[2048];
+  g_snprintf(buf,sizeof(buf), _("Saving %s"), artsub);
+  verbose (buf);
 
   update_work ();
 }
@@ -132,7 +167,7 @@ TaskArticle :: ~TaskArticle ()
   // ensure our on_worker_done() doesn't get called after we're dead
   if (_decoder)
       _decoder->cancel_silently();
-  
+
   _cache.release (_article.get_part_mids());
 }
 
@@ -207,7 +242,7 @@ TaskArticle :: use_nntp (NNTP * nntp)
     needed->buf.clear ();
 
     Quark group;
-    unsigned long number (0ul);
+    uint64_t number (0ul);
     needed->xref.find (nntp->_server, group, number);
     if (number)
       nntp->article (group, number, this);
@@ -291,7 +326,7 @@ TaskArticle :: on_nntp_done  (NNTP             * nntp,
       }
       break;
   }
-  
+
   update_work (nntp);
   check_in (nntp, health);
 }
@@ -310,8 +345,8 @@ TaskArticle :: use_decoder (Decoder* decoder)
   init_steps(100);
   _state.set_working();
   const Article::mid_sequence_t mids (_article.get_part_mids());
-  const ArticleCache :: strings_t filenames (_cache.get_filenames (mids));
-  _decoder->enqueue (this, _save_path, filenames, _save_mode);
+  ArticleCache :: strings_t filenames (_cache.get_filenames (mids));
+  _decoder->enqueue (this, _save_path, filenames, _save_mode, _options, _attachment);
   set_status_va (_("Decoding %s"), _article.subject.c_str());
   debug ("decoder thread was free, enqueued work");
 }
@@ -336,11 +371,20 @@ TaskArticle :: on_worker_done (bool cancelled)
     // now that we're back in the main thread.
 
     foreach_const(Decoder::log_t, _decoder->log_severe, it)
+    {
       Log :: add_err(it->c_str());
+      verbose (it->c_str());
+    }
     foreach_const(Decoder::log_t, _decoder->log_errors, it)
+    {
       Log :: add_err(it->c_str());
+      verbose (it->c_str());
+    }
     foreach_const(Decoder::log_t, _decoder->log_infos, it)
+    {
       Log :: add_info(it->c_str());
+      verbose (it->c_str());
+    }
 
     if (_decoder->mark_read)
       _read.mark_read(_article);
