@@ -27,6 +27,7 @@
 #include <pan/general/worker-pool.h>
 #include <pan/data/article.h>
 #include <pan/data/article-cache.h>
+#include <pan/data/server-info.h>
 #include <pan/data/data.h>
 #include <pan/data/xref.h>
 #include <pan/tasks/nntp.h>
@@ -35,6 +36,7 @@
 namespace pan
 {
   struct Decoder;
+  class Data;
 
   /**
    * Task for downloading, and optionally decoding, articles
@@ -46,6 +48,12 @@ namespace pan
   {
     public: // life cycle
 
+      enum SaveOptions
+      {
+        SAVE_ALL,
+        SAVE_AS
+      };
+
       enum SaveMode { NONE=0, DECODE=(1<<0), RAW=(1<<1) };
 
       TaskArticle (const ServerRank   & server_rank,
@@ -55,11 +63,15 @@ namespace pan
                    ArticleRead        & read,
                    Progress::Listener* l=0,
                    SaveMode             save_mode = NONE,
-                   const Quark        & save_path = Quark());
+                   const Quark        & save_path = Quark(),
+                   const char         * filename=0,
+                   const SaveOptions  & options=SAVE_ALL);
       virtual ~TaskArticle ();
       time_t get_time_posted () const { return _time_posted; }
       const Quark& get_save_path () const { return _save_path; }
+      void  set_save_path (const Quark& q) { _save_path = q;}
       const Article& get_article () const { return _article; }
+      const std::string& get_groups () const { return _groups; }
 
     public: // Task subclass
       unsigned long get_bytes_remaining () const;
@@ -84,21 +96,36 @@ namespace pan
       void on_worker_done (bool cancelled);
 
     protected:
-      const Quark _save_path;
+      Quark _save_path;
       const ServerRank& _server_rank;
       ArticleCache& _cache;
       ArticleRead& _read;
       quarks_t _servers;
       const Article _article;
       const time_t _time_posted;
+      StringView _attachment;
 
     private: // implementation
       const SaveMode _save_mode;
       friend class Decoder;
       Decoder * _decoder;
       bool _decoder_has_run;
+      std::string _groups;
+      const SaveOptions _options;
+
+
+//      typedef std::pair<std::string,StringView> lines_p;
+//      typedef std::vector<lines_p> lines_v;
+//
+//      struct CacheAdder
+//      {
+//        lines_v lines;
+//        void add(std::string& q, const StringView& v)
+//          { lines.push_back(lines_p(q,v)); }
+//      };
+//
+//      CacheAdder adder;
 
-    private:
       struct Needed {
         std::string message_id;
         unsigned long bytes;
