@@ -21,10 +21,17 @@
 #include <cassert>
 #include <cerrno>
 extern "C" {
+  #define PROTOTYPES
+  #include <stdio.h>
+  #include <uulib/uudeview.h>
   #include <glib/gi18n.h>
   #include <gmime/gmime-utils.h>
+  #include <zlib.h>
 }
+#include <fstream>
+#include <iostream>
 #include <pan/general/debug.h>
+#include <pan/general/file-util.h>
 #include <pan/general/macros.h>
 #include <pan/general/messages.h>
 #include <pan/general/utf8-utils.h>
@@ -32,6 +39,7 @@ extern "C" {
 #include "nntp.h"
 #include "task-xover.h"
 
+
 using namespace pan;
 
 namespace
@@ -94,20 +102,22 @@ TaskXOver :: TaskXOver (Data         & data,
   _articles_so_far (0ul),
   _total_minitasks (0)
 {
+
   debug ("ctor for " << group);
 
   // add a ``GROUP'' MiniTask for each server that has this group
+  // initialize the _high lookup table to boundaries
   const MiniTask group_minitask (MiniTask::GROUP);
   quarks_t servers;
   _data.group_get_servers (group, servers);
   foreach_const (quarks_t, servers, it)
-    _server_to_minitasks[*it].push_front (group_minitask);
+    if (_data.get_server_limits(*it))
+    {
+      _server_to_minitasks[*it].push_front (group_minitask);
+      _high[*it] = data.get_xover_high (group, *it);
+    }
   init_steps (0);
 
-  // initialize the _high lookup table to boundaries
-  foreach_const (quarks_t, servers, it)
-    _high[*it] = data.get_xover_high (group, *it);
-
   // tell the users what we're up to
   set_status (group.c_str());
 
@@ -121,6 +131,7 @@ TaskXOver :: ~TaskXOver ()
       _data.set_xover_high (_group, it->first, it->second);
     _data.xover_unref (_group);
   }
+  _data.fire_group_entered(_group, 1, 0);
 }
 
 void
@@ -172,8 +183,8 @@ void
 TaskXOver :: on_nntp_group (NNTP          * nntp,
                             const Quark   & group,
                             unsigned long   qty,
-                            unsigned long   low,
-                            unsigned long   high)
+                            uint64_t        low,
+                            uint64_t        high)
 {
   const Quark& servername (nntp->_server);
 
@@ -188,7 +199,7 @@ TaskXOver :: on_nntp_group (NNTP          * nntp,
          << " low " << low
          << " high " << high);
 
-  unsigned long l(low), h(high);
+  uint64_t l(low), h(high);
   _data.set_xover_low (group, nntp->_server, low);
   //std::cerr << LINE_ID << " This group's range is [" << low << "..." << high << ']' << std::endl;
 
@@ -200,7 +211,7 @@ TaskXOver :: on_nntp_group (NNTP          * nntp,
     l = std::max (low, high+1-_sample_size);
   }
   else { // NEW
-    unsigned long xh (_data.get_xover_high (group, nntp->_server));
+    uint64_t xh (_data.get_xover_high (group, nntp->_server));
     //std::cerr << LINE_ID << " current xover high is " << xh << std::endl;
     l = std::max (xh+1, low);
   }
@@ -209,11 +220,11 @@ TaskXOver :: on_nntp_group (NNTP          * nntp,
   {
     //std::cerr << LINE_ID << " okay, I'll try to get articles in [" << l << "..." << h << ']' << std::endl;
     add_steps (h-l);
-    const int INCREMENT (1000);
+    const int INCREMENT(1000);
     MiniTasks_t& minitasks (_server_to_minitasks[servername]);
-    for (unsigned long m=l; m<=h; m+=INCREMENT) {
+    for (uint64_t m=l; m<=h; m+=INCREMENT) {
       MiniTask mt (MiniTask::XOVER, m, m+INCREMENT);
-      debug ("adding MiniTask for " << servername << ": xover [" << mt._low << '-' << mt._high << ']');
+      debug ("adding MiniTask for " << servername << ": xover [" << mt._low << '-' << mt._high << "]");
       minitasks.push_front (mt);
       ++_total_minitasks;
     }
@@ -240,6 +251,19 @@ namespace
 
     return ul;
   }
+  uint64_t view_to_ull (const StringView& view)
+  {
+    uint64_t ul = 0ul;
+
+    if (!view.empty()) {
+      errno = 0;
+      ul = g_ascii_strtoull (view.str, 0, 10);
+      if (errno)
+        ul = 0ul;
+    }
+
+    return ul;
+  }
 
   bool header_is_nonencoded_utf8 (const StringView& in)
   {
@@ -250,9 +274,10 @@ namespace
 }
 
 void
-TaskXOver :: on_nntp_line (NNTP               * nntp,
-                           const StringView   & line)
+TaskXOver :: on_nntp_line         (NNTP               * nntp,
+                                   const StringView   & line)
 {
+
   pan_return_if_fail (nntp != 0);
   pan_return_if_fail (!nntp->_server.empty());
   pan_return_if_fail (!nntp->_group.empty());
@@ -260,16 +285,31 @@ TaskXOver :: on_nntp_line (NNTP               * nntp,
   _bytes_so_far += line.len;
 
   unsigned int lines=0u;
-  unsigned long number=0ul, bytes=0ul;
-  StringView subj, author, date, mid, ref, tmp, xref, l(line);
+  unsigned long bytes=0ul;
+  uint64_t number=0;
+  StringView subj, author, date, mid, tmp, xref, l(line);
+  std::string ref;
   bool ok = !l.empty();
-  ok = ok && l.pop_token (tmp, '\t');    if (ok) number = view_to_ul (tmp);
+  ok = ok && l.pop_token (tmp, '\t');    if (ok) number = view_to_ull (tmp); tmp.clear();
   ok = ok && l.pop_token (subj, '\t');   if (ok) subj.trim ();
   ok = ok && l.pop_token (author, '\t'); if (ok) author.trim ();
   ok = ok && l.pop_token (date, '\t');   if (ok) date.trim ();
   ok = ok && l.pop_token (mid, '\t');    if (ok) mid.trim ();
-  ok = ok && l.pop_token (ref, '\t');    if (ok) ref.trim ();
-  ok = ok && l.pop_token (tmp, '\t');    if (ok) bytes = view_to_ul (tmp);
+
+  //handle multiple "References:"-message-ids correctly.
+  ok = ok && l.pop_token (tmp, '\t');
+  do
+  {
+    if (tmp.empty()) continue;
+    if (tmp.front() == '<')
+    {
+      tmp.trim();
+      ref += tmp;
+      tmp.clear();
+    } else break;
+  } while ((ok = ok && l.pop_token (tmp, '\t'))) ;
+
+                                         if (ok) bytes = view_to_ul (tmp); tmp.clear();
   ok = ok && l.pop_token (tmp, '\t');    if (ok) lines = view_to_ul (tmp);
   ok = ok && l.pop_token (xref, '\t');   if (ok) xref.trim ();
 
@@ -284,17 +324,21 @@ TaskXOver :: on_nntp_line (NNTP               * nntp,
       || author.empty() // missing author
       || date.empty() // missing date
       || mid.empty() // missing mid
-      || mid.front()!='<' // corrupt mid
-      || (!ref.empty() && ref.front()!='<'))
+      || mid.front()!='<') // corrupt mid
+      /// Concerning bug : https://bugzilla.gnome.org/show_bug.cgi?id=650042
+      /// Even if we didn't get a proper reference here, continue.
+      //|| (!ref.empty() && ref.front()!='<'))
     return;
 
   // if news server doesn't provide an xref, fake one
   char * buf (0);
   if (xref.empty())
-    xref = buf = g_strdup_printf ("%s %s:%lu",
-                       nntp->_server.c_str(), nntp->_group.c_str(), number);
+    xref = buf = g_strdup_printf ("%s %s:%"G_GUINT64_FORMAT,
+                                  nntp->_server.c_str(),
+                                  nntp->_group.c_str(),
+                                  number);
 
-  unsigned long& h (_high[nntp->_server]);
+  uint64_t& h (_high[nntp->_server]);
   h = std::max (h, number);
 
   const char * fallback_charset = NULL; // FIXME
@@ -312,13 +356,13 @@ TaskXOver :: on_nntp_line (NNTP               * nntp,
     nntp->_server, nntp->_group,
     (header_is_nonencoded_utf8(subj) ? subj : header_to_utf8(subj,fallback_charset).c_str()),
     (header_is_nonencoded_utf8(author) ? author : header_to_utf8(author,fallback_charset).c_str()),
-    time_posted, mid, ref, bytes, lines, xref);
+    time_posted, mid, StringView(ref), bytes, lines, xref);
 
   if (article)
     ++_articles_so_far;
 
   // emit a status update
-  int& prev = _last_xover_number[nntp];
+  uint64_t& prev = _last_xover_number[nntp];
   increment_step (number - prev);
   prev = number;
   if (!(_parts_so_far % 500))
@@ -333,7 +377,6 @@ TaskXOver :: on_nntp_done (NNTP              * nntp,
                            Health              health,
                            const StringView  & response UNUSED)
 {
-  //std::cerr << LINE_ID << " nntp " << nntp->_server << " (" << nntp << ") done; checking in.  health==" << health << std::endl;
   update_work (true);
   check_in (nntp, health);
 }
@@ -358,8 +401,8 @@ TaskXOver :: update_work (bool subtract_one_from_nntp_count)
   else if (nntp_count)
     _state.set_working ();
   else {
-    _state.set_completed ();
-    set_finished (OK);
+    _state.set_completed();
+    set_finished(OK);
   }
 }
 
